local utils = require "utils" 
 
Button = {} 
Button.new = function (font, text) 
     
    local this = {} 
    this.text = text 
    this.font = font 
	this.textOverlapped = ""
 
    this.x = -1000
    this.y = 0 

    this.isOutlined = true
    
    this.isClicked   = false
    this.isMouseOver = false
    this.isHeld      = false
    
    this.width  = 200
    this.height = 70
    
    this.color = {}
    
    this.colorStock = {
    	normal = {r = 1.0, g = 1.0, b = 0.3, a = 0.8},
    	over   = {r = 0.5, g = 1.0, b = 0.3, a = 0.8},
    	held   = {r = 1.0, g = 0.3, b = 1.0, a = 0.8}
    }
    this.color = this.colorStock.normal -- 初期色
	
    this.image = nil
    
    -- 更新 
    this.update = function(self) 
    	
		local mouseX, mouseY = love.mouse.getPosition()
		
		self.isClicked = false
		
		-- マウスとの関係判定
		if (    self.x - self.width/2  <= mouseX and mouseX <= self.x + self.width/2 
			and self.y - self.height/2 <= mouseY and mouseY <= self.y + self.height/2 ) then

			self.isMouseOver = true
		else
			self.isMouseOver = false
		end
		
		if ( self.isMouseOver ) then
		
			if ( mousePushed ) then
				self.isHeld = true
			end
			
		else
			self.isClicked   = false
			self.isHeld      = false
		end

		if ( self.isHeld ) then

			if ( mouseReleased ) then
				self.isHeld = false
				self.isClicked = true
			end
		end
		
		
		-- 状態に応じて表示方法を変える
	    self.color = self.colorStock.normal
		
		if ( self.isMouseOver ) then
		    self.color = self.colorStock.over
		end
		if ( self.isHeld ) then
		    self.color = self.colorStock.held
		end
		
    end 
     
    -- 描画 
    this.draw = function(self) 
         
		local r,g,b,a = love.graphics.getColor()
		local currentFont = love.graphics.getFont()

		love.graphics.setColor ( self.color.r, self.color.g, self.color.b, self.color.a )
		love.graphics.rectangle ( "fill", self.x - self.width/2, self.y - self.height/2, self.width, self.height )
		
		if (this.isOutlined) then
			love.graphics.setColor ( 0,0,0,1 )
			love.graphics.rectangle ( "line", self.x - self.width/2, self.y - self.height/2, self.width, self.height )
		end
		
		if ( self.image ~= nil ) then
			
			love.graphics.setColor ( 1,1,1,1 )
			love.graphics.draw ( self.image, self.x, self.y, math.rad(0), 1, 1, self.image:getWidth()/2, self.image:getHeight()/2 )
			
		end
		
		
		love.graphics.setColor ( 0,0,0,1 )
		love.graphics.setFont ( self.font )
		
		local textOffsetX = self.font:getWidth  ( self.text ) / 2
		local textOffsetY = self.font:getHeight ( self.text ) / 2
		love.graphics.print( self.text, self.x, self.y, math.rad(0), 1,1, textOffsetX, textOffsetY )
		
		if ( self.textOverlapped ~= "" ) then
			love.graphics.print( self.textOverlapped, self.x, self.y, math.rad(0), 1,1, textOffsetX, textOffsetY )
		end
		
		love.graphics.setColor ( r,g,b,a )
		love.graphics.setFont(currentFont)
         
    end 
     
     
     
    return this 
end 
