--=========================================================
-- エル本体
--=========================================================
Elu = {}
Elu.new = function ( world, x, y )
	
	local this = {}
	
	this.x = x
	this.y = y
	this.world = world
	
	this.object1 = nil
	this.object2 = nil
	
	this.frameCount = 0
	
	this.r = math.random ( 0.0, 1.0)
	this.g = math.random ( 0.0, 1.0)
	this.b = math.random ( 0.0, 1.0)
	
	
	this.angleDegree = math.random ( 0, 360 )
	
	
	
	
	--=========================================================
	-- 本体出現 
	--=========================================================
	this.putEntity = function (self)

		this.object1 = {}
		this.object1.body = love.physics.newBody ( this.world, this.x-25, this.y-50, "dynamic")
		this.object1.body:setUserData ( {obj = this, name = "elu1"} )
	
		this.object1.shape   = love.physics.newRectangleShape ( 20, 100 )
	
		this.object1.fixture = love.physics.newFixture ( this.object1.body, this.object1.shape, 10) -- 3つめの値：dencity
		this.object1.fixture:setFriction(1.0) -- 0.0 - 1.0
		this.object1.fixture:setRestitution (0.3)

		this.object2 = {}
		this.object2.body = love.physics.newBody ( this.world, this.x + 200, this.y + 80, "dynamic")
		this.object2.body:setUserData ( {obj = this, name = "elu2"} )
	
		this.object2.shape   = love.physics.newRectangleShape ( 50, 20 )
	
		this.object2.fixture = love.physics.newFixture ( this.object2.body, this.object2.shape, 10) -- 3つめの値：dencity
		this.object2.fixture:setFriction(1.0) -- 0.0 - 1.0
		this.object2.fixture:setRestitution (0.3)

		this.joint = love.physics.newWeldJoint( -- body1, body2, x1, y1, x2, y2, collideConnected )
			this.object1.body, this.object2.body,
			this.object1.body:getX() + 10, this.object1.body:getY() + 40,
			this.object2.body:getX() - 25, this.object2.body:getY() + 0,
			false
		)

	end

	--=========================================================
	-- デストラクト
	--=========================================================
	this.destruct = function (self)
		
		if ( this.object1 ~= nil ) then
			destroyBodyReserveList [#destroyBodyReserveList + 1]  = this.object1.body;
		end
		if ( this.object2 ~= nil ) then
			destroyBodyReserveList [#destroyBodyReserveList + 1]  = this.object2.body;
		end
		
	end
	
	--=========================================================
	-- 消滅してパーティクルを発する
	--=========================================================
	this.banish = function (self)
		
		local isBanished = false
		
		if ( this.object1 ~= nil ) then
			
			for n = 1, 3, 1 do
				addParticle ( math.random(-10,10), math.random(-20, 0), this.object1.body:getX(), this.object1.body:getY(), { r=this.r, g=this.g, b=this.b } )
			end
			
			isBanished = true
			
			
		end
		
		if ( this.object2 ~= nil ) then

			for n = 1, 2, 1 do
				addParticle ( math.random(-10,10), math.random(-20, 0), this.object2.body:getX(), this.object2.body:getY(), { r=this.r, g=this.g, b=this.b } )
			end
			
			isBanished = true
			
		end
		
		
		-- アラート段階だったものは消さない
		if ( isBanished ) then
		
			--this.object1.body:destroy()
			--this.object2.body:destroy()
			
			destroyBodyReserveList [#destroyBodyReserveList + 1]  = this.object1.body;
			destroyBodyReserveList [#destroyBodyReserveList + 1]  = this.object2.body;

			deleteElu ( this )
		end
		
	end

	--=========================================================
	-- 位置セット
	--=========================================================
	this.setPosition = function(self, x, y)
		
		self.x = x
		self.y = y
		
		if ( this.object1 ~= nil and this.object2 ~= nil ) then
			self.object1.body:setX(x)
			self.object1.body:setY(y)
		end
		
	end
	
	
	--=========================================================
	-- 更新
	--=========================================================
	this.update = function(self)
		
		if ( this.object1 ~= nil and this.object2 ~= nil ) then
			self.x = self.object1.body:getX()
			self.y = self.object1.body:getY()
		else
			
			this.frameCount = this.frameCount + 1
			
			
			
			if ( this.frameCount > 50 ) then
				this:putEntity()
			end
			
		end
		
	end
	
	
	
	--=========================================================
	-- 描画
	--=========================================================
	this.draw = function(self)
		
		local r, g, b, a = love.graphics.getColor()

		if ( this.object1 ~= nil and this.object2 ~= nil ) then
		
		
			love.graphics.setColor( this.r, this.g, this.b )
		
			love.graphics.polygon ( "fill", this.object1.body:getWorldPoints ( this.object1.shape:getPoints() ) )
			love.graphics.polygon ( "fill", this.object2.body:getWorldPoints ( this.object2.shape:getPoints() ) )
			
			
		else
		
			love.graphics.setColor( 1,1,1 )
			
			local f = "0"..tostring ( this.frameCount*2 )
			f = string.sub (f, string.len (f)-1, string.len (f)-1)
			local nf = tonumber(f)
			
			if ( nf%2 == 0 ) then
			
				--local screenX, screenY = camera:toScreen ( this.x, this.y  );
				local dummy, alertY = camera:toWorld ( 0, 32 )
				
				love.graphics.draw ( alert, this.x, alertY )
			end
		end


		love.graphics.setColor(r, g, b, a)
	end
	
	
		
	return this
end

