--=========================================================
-- 功績＿ノーダメージで三原色連続達成１
--=========================================================
FeatMonitor_nodamageRGB1 = {}
FeatMonitor_nodamageRGB1.new = function ()
	
	local this = {}
	
	this.id = "nodamageRGB1"
	this.alreadyAchieved = featInfo[this.id]
	
	this.message = ""
	this.messageCount = 0
	
	this.iconImage = love.graphics.newImage ( "featIcon_kirakira.png" )
	
	this.drawX = 0
	this.drawY = 0
	this.isMouseover = false
	
	this.goalValue = 10
	
	this.name = "無傷の三原色クイーン１"
	this.description = "ノーダメージで三原色を"..this.goalValue.."回発動する。"
	
	
	this.phase = ""
	this.preHp = 0
	this.preInvocation = nil
	this.rgbCount = 0
	
	--=========================================================
	-- 最初の処理
	--=========================================================
	this.init = function(self)
		
		this.preHp = hp
		this.rgbCount = 0
		this.preInvocation = nil
		
		this.phase = "notYetAccomplished"
		
		
	end
	this:init()
	
	--=========================================================
	-- 更新
	--=========================================================
	this.update = function(self)
		
		if ( this.messageCount > 0) then
			this.messageCount = this.messageCount - 1
		end
		
		-- 達成済みであれば何もしない
		if ( this.alreadyAchieved ) then return end
		
		
		-- 未達成
		if ( this.phase == "notYetAccomplished") then
			
			-- ダメージを受けていた（前回フレームよりHPが減っていた）
			if ( this.preHp > hp) then
			
				-- カウントをリセット
				this.rgbCount = 0
			end

			-- 発動していた
			if ( this.preInvocation == nil and invocation ~= nil ) then
				this.rgbCount = this.rgbCount + 1
			end
			
			if ( this.rgbCount >= this.goalValue ) then
				this.phase = "accomplishedYetNotAnounced"
			end

		-- 達成：発動カットインが終了するのを待ってアナウンス
		elseif ( this.phase == "accomplishedYetNotAnounced") then
			
			if ( invocation == nil ) then
			
				this.message = "["..this.name.."]を達成！"
				this.messageCount = 100
			
				this.alreadyAchieved = true
				featInfo[this.id] = true
				playFeatAchievedFanfare ()
				
				this.phase = "allAccomplished"
			end
			
		elseif ( this.phase == "allAccomplished") then
			
			-- 処理なし
			
		end
		
		-- モニタリング用の値を更新
		this.preHp = hp
		this.preInvocation = invocation
		
		
	end
	
	--=========================================================
	-- 描画
	--=========================================================
	this.draw = function(self)
		
		local r, g, b, a = love.graphics.getColor()
		
		if ( this.messageCount > 0) then
		
			drawText ( this.message, 10, screenHeight - 50, {r=1,g=1,b=0})
			
		end
		
		
		love.graphics.setColor(r, g, b, a)
	end
	
	

	--=========================================================
	-- 更新：アイコンとして
	--=========================================================
	this.updateAsIcon = function(self)
		
		local width = 100
		local height = 100
		
		this.isMouseover = false
		local mouseX, mouseY = love.mouse.getPosition()
		
		if ( 
			this.drawX < mouseX and mouseX < this.drawX + width 
			and
			this.drawY < mouseY and mouseY < this.drawY + height 
		) 
		then
			this.isMouseover = true
		end
		
		
	end
	--=========================================================
	-- 描画：アイコンとして
	--=========================================================
	this.drawAsIcon = function(self, x, y)
		
		this.drawX = x
		this.drawY = y
		
		local r, g, b, a = love.graphics.getColor()
		
		
		--love.draw.rectangle("fill", x, y, 100, 100)
		--love.draw.rectangle("line", x, y, 100, 100)
		
		if ( this.isMouseover ) then
		
			love.graphics.setColor (1,1,1,0.8)
			love.graphics.rectangle ( "fill", x, y, 100, 100 )	
			
		end
		
		
		if ( this.alreadyAchieved ) then
			love.graphics.setColor (1,1,1,1)
			love.graphics.draw ( this.iconImage, x, y )
		else
			
			local currentShader = love.graphics.getShader ()
			love.graphics.setShader ( shader.grayscale )
			love.graphics.setColor (1.0,1.0,1.0,0.5)
			love.graphics.draw ( this.iconImage, x, y )
			love.graphics.setShader ( currentShader )
		end
		
		
		love.graphics.setColor(r, g, b, a)
	end
	

		
	return this
end

