local utils = require "utils"

--=========================================================
-- 功績表示ウィンドウ
--=========================================================
FeatWindow = {}
FeatWindow.new = function ()
	
	local this = {}
	
	this.x = screenWidth
	this.y = 60
	
	this.width  = 540
	this.height = 410
	
	
	this.buttonClose = Button.new( fonts.system, "×")
	this.buttonClose.width  = 50
	this.buttonClose.height = 50
	
	this.phase = ""
	
	this.openCloseSound = love.audio.newSource ("on04.wav", "static") -- 連打はしないだろうという想定。
	
	--=========================================================
	-- 最初の処理
	--=========================================================
	this.init = function(self)
		
		--this.buttonClose.colorStock = {
		--	normal = {r = 1.0, g = 1.0, b = 0.3, a = 0.8},
		--	over   = {r = 0.5, g = 1.0, b = 0.3, a = 0.8},
		--	held   = {r = 1.0, g = 0.3, b = 1.0, a = 0.8}
		--}		
		this.phase = "opening"
		
		love.audio.play(this.openCloseSound)
		
	end
	this:init()
	
	--=========================================================
	-- 更新
	--=========================================================
	this.update = function(self)
		
		local result = ""
		

		for i = 1, #featMonitorList, 1 do
			featMonitorList[i]:updateAsIcon( iconX, iconY )
		end		
		
		
		if ( this.phase == "opening" ) then
			
			local goalX = (screenWidth - this.width)/2
			this.x = utils.Approach ( this.x, goalX, 30)
		
			this.buttonClose.x = this.x + this.width - this.buttonClose.width/2 - 5
			this.buttonClose.y = this.y + this.buttonClose.height/2 + 5
			
			this.buttonClose:update()
			
			if ( this.x <= goalX ) then
				this.phase = "normal"
			end
		
		elseif ( this.phase == "normal" ) then
		
			this.buttonClose:update()

			if (self.buttonClose.isClicked) then
				this.phase = "closing"
				love.audio.play(this.openCloseSound)
			end	
			
		elseif ( this.phase == "closing" ) then

			this.x = utils.Approach ( this.x, screenWidth, 30)
		
			this.buttonClose.x = this.x + this.width - this.buttonClose.width/2 - 5
			this.buttonClose.y = this.y + this.buttonClose.height/2 + 5
			
			if ( this.x >= screenWidth ) then
				this.phase = "end"
			end
			
		elseif ( this.phase == "end" ) then
			result = "done"
		end
		
		
		return result
	end
	this:update()
	
	
	--=========================================================
	-- 描画
	--=========================================================
	this.draw = function(self)
		
		local r, g, b, a = love.graphics.getColor()
		
		love.graphics.setColor ( 0.3,0.8,0.3, 0.8 )
		love.graphics.rectangle ( "fill", this.x, this.y, this.width, this.height)
		
		this.buttonClose:draw()
		
		drawText ( "===称号リスト===", this.x + 150, this.y + 10, {r=0,g=0,b=1} )
		
		-- 功績の説明文の下地
		--love.graphics.setColor ( 0.6,1.0,0.6, 0.8 )
		love.graphics.setColor(0.2, 0.5, 0.20)
		love.graphics.rectangle ( "fill", this.x + 10, this.y + 295, this.width - 20, 105)
		
		
		local iconX = this.x + 10
		local iconY = this.y + 70
		for i = 1, #featMonitorList, 1 do
			
			featMonitorList[i]:drawAsIcon( iconX, iconY )
			
			if ( featMonitorList[i].isMouseover ) then
				
				local name = featMonitorList[i].name
				if ( not featMonitorList[i].alreadyAchieved ) then
					name = name.."【未】"
				end
				
				drawText ( name, this.x + 20, this.y + 300, {r=1,g=1,b=1}, font )
				drawText ( featMonitorList[i].description, this.x + 20, this.y + 350, {r=1,g=1,b=1}, fontSmall )
				
			end
			
			iconX = iconX + 100
			
			if ( i == 5 ) then
				iconX = this.x + 10
				iconY = iconY + 120
			end
		end
		
		
		
		love.graphics.setColor(r, g, b, a)
	end
	
	
		
	return this
end

