local utils = require "utils"

--require( "sub1" )

--=========================================================
-- 説明ウィンドウ
--=========================================================
InstructionWindow = {}
InstructionWindow.new = function ()
	
	local this = {}
	
	this.x = screenWidth
	this.y = 60
	
	this.width  = 540
	this.height = 410
	
	
	this.buttonClose = Button.new( fonts.system, "×")
	this.buttonClose.width  = 50
	this.buttonClose.height = 50

	this.buttonNext = Button.new( fonts.system, "次->")
	this.buttonNext.width  = 100
	this.buttonNext.height = 60

	this.buttonPrev = Button.new( fonts.system, " <-閉 ")
	this.buttonPrev.width  = 100
	this.buttonPrev.height = 60
	
	this.pageIndex = 0
	this.pageList = {}
	
	this.phase = ""
	
	this.openCloseSound = love.audio.newSource ("on04.wav", "static") -- 連打はしないだろうという想定。
	
	this.image = {}
	this.image.instruction1_1   = love.graphics.newImage ("instruction1_1.png")
	this.image.instruction1_2   = love.graphics.newImage ("instruction1_2.png")
	this.image.instruction1_3_1 = love.graphics.newImage ("instruction1_3_1.png")
	this.image.instruction1_3_2 = love.graphics.newImage ("instruction1_3_2.png")
	this.image.instruction1_3_3 = love.graphics.newImage ("instruction1_3_3.png")
	this.image.instruction2_1   = love.graphics.newImage ("instruction2_1.png")
	this.image.instruction2_2   = love.graphics.newImage ("instruction2_2.png")
	this.image.instruction3_1   = love.graphics.newImage ("instruction3_1.png")
	this.image.instruction4_1   = love.graphics.newImage ("instruction4_1.png")
	this.image.instruction5_1   = love.graphics.newImage ("instruction5_1.png")
	this.image.instruction6_1   = love.graphics.newImage ("instruction6_1.png")
	this.image.instruction7_1   = love.graphics.newImage ("instruction7_1.png")
	this.image.instruction7_1_1 = love.graphics.newImage ("instruction7_1_1.png")
	this.image.instruction7_2   = love.graphics.newImage ("instruction7_2.png")
	
	--=========================================================
	-- 最初の処理
	--=========================================================
	this.init = function(self)
		
		--this.buttonClose.colorStock = {
		--	normal = {r = 1.0, g = 1.0, b = 0.3, a = 0.8},
		--	over   = {r = 0.5, g = 1.0, b = 0.3, a = 0.8},
		--	held   = {r = 1.0, g = 0.3, b = 1.0, a = 0.8}
		--}		
		this.phase = "opening"
		
		love.audio.play(this.openCloseSound)
		
		this.pageIndex = 1
		this.pageList [ #this.pageList + 1 ] = this.drawPage_1
		this.pageList [ #this.pageList + 1 ] = this.drawPage_2
		this.pageList [ #this.pageList + 1 ] = this.drawPage_2_plus1
		this.pageList [ #this.pageList + 1 ] = this.drawPage_3
		this.pageList [ #this.pageList + 1 ] = this.drawPage_4
		this.pageList [ #this.pageList + 1 ] = this.drawPage_5
		this.pageList [ #this.pageList + 1 ] = this.drawPage_6
		this.pageList [ #this.pageList + 1 ] = this.drawPage_7
		this.pageList [ #this.pageList + 1 ] = this.drawPage_7_plus1
		this.pageList [ #this.pageList + 1 ] = this.drawPage_8
		this.pageList [ #this.pageList + 1 ] = this.drawPage_9
		this.pageList [ #this.pageList + 1 ] = this.drawPage_10
		this.pageList [ #this.pageList + 1 ] = this.drawPage_10_plus1
		this.pageList [ #this.pageList + 1 ] = this.drawPage_11
		this.pageList [ #this.pageList + 1 ] = this.drawPage_11_plus1
		this.pageList [ #this.pageList + 1 ] = this.drawPage_12
		this.pageList [ #this.pageList + 1 ] = this.drawPage_12_plus1
		this.pageList [ #this.pageList + 1 ] = this.drawPage_13
		this.pageList [ #this.pageList + 1 ] = this.drawPage_14
		
	end
	
	--=========================================================
	-- 更新
	--=========================================================
	this.update = function(self)
		
		local result = ""
		

		for i = 1, #featMonitorList, 1 do
			featMonitorList[i]:updateAsIcon( iconX, iconY )
		end	
		
		-- ボタンの位置をウィンドウに合わせる		
		local buttonPosToWindow = function()
			self.buttonClose.x = self.x + self.width - self.buttonClose.width/2 - 5
			self.buttonClose.y = self.y + self.buttonClose.height/2 + 5

			self.buttonNext.x = self.x + self.width/2 + self.buttonNext.width/2 + 10
			self.buttonNext.y = self.y + self.height - self.buttonNext.height/2 - 5

			self.buttonPrev.x = self.x + self.width/2 - self.buttonPrev.width/2 - 10
			self.buttonPrev.y = self.y + self.height - self.buttonPrev.height/2 - 5
		end
		
		
		if ( this.phase == "opening" ) then
			
			local goalX = (screenWidth - this.width)/2
			this.x = utils.Approach ( this.x, goalX, 30)
			
			buttonPosToWindow()		
			
			this.buttonClose:update()
			this.buttonNext:update()
			this.buttonPrev:update()
			
			if ( this.x <= goalX ) then
				this.phase = "normal"
			end
		
		elseif ( this.phase == "normal" ) then
		
			this.buttonClose:update()
			this.buttonNext:update()
			this.buttonPrev:update()
			
			-- 閉じるをクリック
			if (self.buttonClose.isClicked) then
				this.phase = "closing"
				love.audio.play(this.openCloseSound)
			end	
			
			-- 次へをクリック
			if (self.buttonNext.isClicked) then
				
				if ( this.pageIndex < #this.pageList ) then
					this.pageIndex = this.pageIndex + 1
					local s = love.audio.newSource ("book_pageflip01.mp3", "static")
					love.audio.play(s)
					
					if ( this.pageIndex == #this.pageList ) then
						self.buttonNext.text = "終->"
					else
						self.buttonNext.text = "次->"
					end

					if ( this.pageIndex == 1 ) then
						self.buttonPrev.text = " <-閉 "
					else
						self.buttonPrev.text = " <-戻 "
					end
					
				else
					this.phase = "closing"
					love.audio.play(this.openCloseSound)
				end
			end	

			--戻るをクリック
			if (self.buttonPrev.isClicked) then
				
				if ( this.pageIndex > 1 ) then
					this.pageIndex = this.pageIndex - 1
					local s = love.audio.newSource ("book_pageflip01.mp3", "static")
					love.audio.play(s)

					if ( this.pageIndex == #this.pageList ) then
						self.buttonNext.text = "終->"
					else
						self.buttonNext.text = "次->"
					end

					if ( this.pageIndex == 1 ) then
						self.buttonPrev.text = " <-閉 "
					else
						self.buttonPrev.text = " <-戻 "
					end
					
				else
					this.phase = "closing"
					love.audio.play(this.openCloseSound)
				end
			end	
			
		elseif ( this.phase == "closing" ) then

			this.x = utils.Approach ( this.x, screenWidth, 30)
		
			buttonPosToWindow()		
			
			if ( this.x >= screenWidth ) then
				this.phase = "end"
			end
			
		elseif ( this.phase == "end" ) then
			result = "done"
		end
		
		
		return result
	end
	this:update()
	
	
	--=========================================================
	-- 描画
	--=========================================================
	this.draw = function(self)
		
		local r, g, b, a = love.graphics.getColor()
		
		-- ウィンドウとしての下地
		love.graphics.setColor ( 0.3,0.8,0.3, 0.8 )
		love.graphics.rectangle ( "fill", this.x, this.y, this.width, this.height)
		
		-- 各種ボタン
		this.buttonClose:draw()
		this.buttonNext:draw()
		this.buttonPrev:draw()
		
		-- 現在のページを描画
		love.graphics.setColor ( 1,1,1,1 )
		this.pageList [ this.pageIndex ]()
		
		-- 現在のページ番号を描画
		local pageIndexInfo = "("..tostring(this.pageIndex).."/"..tostring(#this.pageList)..")"
		drawText ( pageIndexInfo, this.x + 5, this.y + 5 )
		
		love.graphics.setColor(r, g, b, a)
	end
	
	--=========================================================
	-- ページを描画:1
	--=========================================================
	this.drawPage_1 = function(self)
		
		love.graphics.draw ( this.image.instruction1_1, this.x + this.width/2 - 100, this.y + 120)

		drawText ( "画面の好きな場所を左クリック。", this.x + 10, this.y + 70, {r=1, g=1, b=1} )
		
		
	end
	--=========================================================
	-- ページを描画:2
	--=========================================================
	this.drawPage_2 = function(self)
		
		love.graphics.draw ( this.image.instruction1_2, this.x + this.width/2 - 100, this.y + 120)
		drawText ( "ワイヤー（綱[つな]）を出して", this.x + 10, this.y + 70, {r=1, g=1, b=1} )
		drawText ( "ステージを飛び回れ！", this.x + 200, this.y + 280, {r=1, g=1, b=1} )
		
	end
	this.drawPage_2_plus1 = function(self)
		
		local x1 = this.x + this.width/2 - 100 - 170
		local x2 = this.x + this.width/2 - 100
		local x3 = this.x + this.width/2 - 100 + 170
		
		local currentFont = love.graphics.getFont()
		
		love.graphics.draw ( this.image.instruction1_3_1, x1, this.y + 90)
		love.graphics.draw ( this.image.instruction1_3_2, x2, this.y + 90)
		love.graphics.draw ( this.image.instruction1_3_3, x3, this.y + 90)
		
		local text1 = "押す！"
		local text2 = "押しっぱなし！"
		local text3 = "離す！"
		
		drawText ( text1, x1 - currentFont:getWidth(text1)/2 + 100, this.y + 70, {r=1, g=1, b=1} )
		drawText ( text2, x2 - currentFont:getWidth(text2)/2 + 100, this.y + 280, {r=1, g=1, b=1} )
		drawText ( text3, x3 - currentFont:getWidth(text3)/2 + 100, this.y + 200, {r=1, g=1, b=1} )
		
	end
	--=========================================================
	-- ページを描画:3
	--=========================================================
	this.drawPage_3 = function(self)
		
		drawText ( "上から降るエルに、", this.x + 10, this.y + 50, {r=1, g=1, b=1} )

		love.graphics.draw ( this.image.instruction2_1, this.x + this.width/2 - 180, this.y + 90)
		love.graphics.draw ( this.image.instruction2_2, this.x + this.width/2 - 20, this.y + 150)
		

		drawText ( "当たらないよう気を付けながら……", this.x + 10, this.y + 300, {r=1, g=1, b=1} )
		
	end
	--=========================================================
	-- ページを描画:4
	--=========================================================
	this.drawPage_4 = function(self)
		
		love.graphics.draw ( this.image.instruction3_1, this.x + this.width/2 - 100, this.y + 120)

		drawText ( "三色の光を集めよう。", this.x + 10, this.y + 70, {r=1, g=1, b=1} )
		
		
	end
	--=========================================================
	-- ページを描画:5
	--=========================================================
	this.drawPage_5 = function(self)
		
		love.graphics.draw ( this.image.instruction4_1, this.x + this.width/2 - 100, this.y + 120)

		drawText ( "赤・緑・青の三色がそろうと……", this.x + 10, this.y + 70, {r=1, g=1, b=1} )
		
		local lw = love.graphics.getLineWidth()
		love.graphics.setLineWidth(4)

		love.graphics.setLineStyle("smooth")

		love.graphics.setColor(0.5, 1.0, 0.5)
		love.graphics.ellipse("line", this.x + 270, this.y + 190, 120, 40)

		love.graphics.setLineWidth(lw)
		
	end
	--=========================================================
	-- ページを描画:6
	--=========================================================
	this.drawPage_6 = function(self)
		
		love.graphics.draw ( this.image.instruction5_1, this.x + this.width/2 - 260, this.y + this.height/2 - 180)

		drawText ( "「三原色」が発動して高得点！", this.x + 10, this.y + 70, {r=1, g=1, b=1} )
		
		-- 各種ボタン
		this.buttonClose:draw()
		this.buttonNext:draw()
		this.buttonPrev:draw()
	end
	--=========================================================
	-- ページを描画:7
	--=========================================================
	this.drawPage_7 = function(self)
		
		love.graphics.draw ( this.image.instruction6_1, this.x + this.width/2 - 260, this.y + this.height/2 - 180)

		drawText ( "降り積もったエルも消滅だ！", this.x + 10, this.y + 70, {r=1, g=1, b=1} )

		
		-- 各種ボタン
		this.buttonClose:draw()
		this.buttonNext:draw()
		this.buttonPrev:draw()
		
		
	end
	this.drawPage_7_plus1 = function(self)
		
		love.graphics.draw ( this.image.instruction6_1, this.x + this.width/2 - 260, this.y + this.height/2 - 180)

		drawText ( "発生した小さな光の粒も、", this.x + 10, this.y + 70, {r=1, g=1, b=1} )

		drawText ( "取るとスコアに。", this.x + 250, this.y + 220, {r=1, g=1, b=1} )
		
		-- 各種ボタン
		this.buttonClose:draw()
		this.buttonNext:draw()
		this.buttonPrev:draw()
		
		
	end
	--=========================================================
	-- ページを描画:8
	--=========================================================
	this.drawPage_8 = function(self)
		
		drawText ( "ずんだ餅を取ると体力回復。", this.x + 10, this.y + 100, {r=1, g=1, b=1} )
		
		love.graphics.setColor(0.5, 0.5, 1.0)
		love.graphics.rectangle ( "fill", this.x + this.width/2 - 100, this.y + this.height/2 - 70 + 20, 200, 140 )
		
		love.graphics.setColor(1.0, 1.0, 1.0)
		love.graphics.draw ( zunda, this.x + this.width/2 - 16, this.y + this.height/2 - 16 + 20 )
		
	end
	--=========================================================
	-- ページを描画:9
	--=========================================================
	this.drawPage_9 = function(self)
		
		drawText ( "ずんだ餅は「三原色」発動の後、", this.x + 10, this.y + 100, {r=1, g=1, b=1} )

		love.graphics.setColor(0.5, 0.5, 1.0)
		love.graphics.rectangle ( "fill", this.x + this.width/2 - 100, this.y + this.height/2 - 70 + 20, 200, 140 )
		
		love.graphics.setColor(1.0, 1.0, 1.0)
		love.graphics.draw ( zunda, this.x + this.width/2 - 16, this.y + this.height/2 - 16 + 20 )
		
		drawText ( "画面のどこかに１つだけ出現するよ。", this.x + 10, this.y + 300, {r=1, g=1, b=1} )
		
	end
	--=========================================================
	-- ページを描画:10
	--=========================================================
	this.drawPage_10 = function(self)
		
	
		love.graphics.draw ( this.image.instruction7_1, this.x + 50, this.y + 140 ) 

		drawText ( "三原色が発動するたびに", this.x + 10, this.y + 50, {r=1, g=1, b=1} )
		drawText ( "レベルが上がって、", this.x + 250, this.y + 90, {r=1, g=1, b=1} )

		
		local lw = love.graphics.getLineWidth()
		love.graphics.setLineWidth(4)

		love.graphics.setLineStyle("smooth")

		love.graphics.setColor(0.5, 1.0, 0.5)
		love.graphics.ellipse("line", this.x + 185, this.y + 255, 40, 30)

		love.graphics.setLineWidth(lw)
		
	end
	this.drawPage_10_plus1 = function(self)
		
	
		love.graphics.draw ( this.image.instruction7_1, this.x + 50, this.y + 140 ) 

		--love.graphics.draw ( this.image.instruction2_1, this.x + this.width/2 - 180, this.y + 90)
		love.graphics.setColor(1.0, 1.0, 1.0,0.7)
		love.graphics.draw ( this.image.instruction2_2, this.x + 300, this.y + 110 )		
		love.graphics.draw ( this.image.instruction2_2, this.x + 320, this.y + 150 )		
		love.graphics.draw ( this.image.instruction2_2, this.x + 340, this.y + 190 )		

		drawText ( "三原色が発動するたびに", this.x + 10, this.y + 50, {r=1, g=1, b=1} )
		drawText ( "レベルが上がって、", this.x + 250, this.y + 90, {r=1, g=1, b=1} )

		drawText ( "エルが降る頻度が上がります。", this.x + 10, this.y + 300, {r=1, g=1, b=1} )
		
	end
	--=========================================================
	-- ページを描画:11
	--=========================================================
	this.drawPage_11 = function(self)
		
		love.graphics.draw ( this.image.instruction7_1, this.x + 50, this.y + 140 ) 

		drawText ( "レベルの右側の数字は", this.x + 10, this.y + 50, {r=1, g=1, b=1} )
		drawText ( "光を取るたび上昇。", this.x + 250, this.y + 90, {r=1, g=1, b=1} )

		
		local lw = love.graphics.getLineWidth()
		love.graphics.setLineWidth(4)

		love.graphics.setLineStyle("smooth")

		love.graphics.setColor(0.5, 1.0, 0.5)
		love.graphics.ellipse("line", this.x + 185, this.y + 255, 40, 30)

		love.graphics.setColor(1.0, 0.5, 0.5)
		love.graphics.line(this.x + 190, this.y + 270, this.x + 220, this.y + 270)

		love.graphics.setLineWidth(lw)
		
	end
	this.drawPage_11_plus1 = function(self)
		
		love.graphics.draw ( this.image.instruction7_1_1, this.x + 50, this.y + 140 ) 

		drawText ( "三色そろったらリセットされます。", this.x + 10, this.y + 50, {r=1, g=1, b=1} )
		
		local lw = love.graphics.getLineWidth()
		love.graphics.setLineWidth(4)

		love.graphics.setLineStyle("smooth")

		love.graphics.setColor(0.5, 1.0, 0.5)
		love.graphics.ellipse("line", this.x + 185, this.y + 255, 40, 30)

		love.graphics.setColor(1.0, 0.5, 0.5)
		love.graphics.line(this.x + 190, this.y + 270, this.x + 220, this.y + 270)

		love.graphics.setLineWidth(lw)


		
	end


	--=========================================================
	-- ページを描画:12
	--=========================================================
	this.drawPage_12 = function(self)
		
		love.graphics.draw ( this.image.instruction7_1, this.x + 50, this.y + 140 ) 

		drawText ( "この数字が増えるほど、", this.x + 10, this.y + 50, {r=1, g=1, b=1} )
		
		
		local lw = love.graphics.getLineWidth()
		love.graphics.setLineWidth(4)

		love.graphics.setLineStyle("smooth")

		love.graphics.setColor(1.0, 0.5, 0.5)
		love.graphics.line(this.x + 190, this.y + 270, this.x + 220, this.y + 270)

		love.graphics.setLineWidth(lw)		
	end
	this.drawPage_12_plus1 = function(self)
		
		love.graphics.draw ( this.image.instruction7_1, this.x + 50, this.y + 140 ) 

		love.graphics.draw ( this.image.instruction7_2, this.x + 230, this.y + 160 ) 

		drawText ( "この数字が増えるほど、", this.x + 10, this.y + 50, {r=1, g=1, b=1} )
		drawText ( "エルが同時にたくさん降るよ。", this.x + 100, this.y + 100, {r=1, g=1, b=1} )


		local lw = love.graphics.getLineWidth()
		love.graphics.setLineWidth(4)

		love.graphics.setLineStyle("smooth")

		love.graphics.setColor(1.0, 0.5, 0.5)
		love.graphics.line(this.x + 190, this.y + 270, this.x + 220, this.y + 270)

		love.graphics.setLineWidth(lw)		


		-- 各種ボタン
		this.buttonClose:draw()
		this.buttonNext:draw()
		this.buttonPrev:draw()		
	end
	--=========================================================
	-- ページを描画:13
	--=========================================================
	this.drawPage_13 = function(self)
		
		love.graphics.draw ( this.image.instruction3_1, this.x + this.width/2 - 100, this.y + 90)

		drawText ( "三色をまんべんなく", this.x + 10, this.y + 70, {r=1, g=1, b=1} )

		drawText ( "取っていくのが安全！", this.x + 220, this.y + 280, {r=1, g=1, b=1} )
	end
	--=========================================================
	-- ページを描画:14
	--=========================================================
	this.drawPage_14 = function(self)
		
		love.graphics.setColor(0.2, 0.5, 0.20)
		love.graphics.rectangle ( "fill", this.x + 10, this.y + 80, this.width - 20, 250 )

		drawText ( "最後に……", this.x + 20, this.y + 90, {r=1, g=1, b=1}, fontSmall )
		
		drawText ( "===ショートカットキー===", this.x + 100, this.y + 110, {r=1, g=1, b=1} )

		drawText ( "F9：FPS表示/非表示", this.x + 30, this.y + 180, {r=1, g=1, b=1} )

		drawText ( "F11：フルスクリーン/ウィンドウ", this.x + 30, this.y + 240, {r=1, g=1, b=1} )

	end
	
	this:init()
	
	return this
end

