local utils = require "utils"
--=========================================================
-- 光を取って入れておくところ
--=========================================================
LightBox = {}
LightBox.new = function ()
	
	local this = {}
	
	this.lightRed   = love.graphics.newQuad(128, 128, 64, 64, lightSheet:getDimensions() )
	this.lightGreen = love.graphics.newQuad(128,  64, 64, 64, lightSheet:getDimensions() )
	this.lightBlue  = love.graphics.newQuad(128,   0, 64, 64, lightSheet:getDimensions() )
	
	
	this.redNum   = 0
	this.greenNum = 0
	this.blueNum  = 0
	
	
	this.toBeScoredLightList = {}
	
	--=========================================================
	-- 初期化
	--=========================================================
	this.init = function ()
		
		this.redNum   = 0
		this.greenNum = 0
		this.blueNum  = 0
		
		this.toBeScoredLightList = {}
		
	end
	
	--=========================================================
	-- 光を入れる
	--=========================================================
	this.putLight = function(self, colorName)
		
		if ( colorName == "red"   ) then this.redNum   = this.redNum   + 1 end
		if ( colorName == "green" ) then this.greenNum = this.greenNum + 1 end
		if ( colorName == "blue"  ) then this.blueNum  = this.blueNum  + 1 end
		
		
		-- 三色そろった		
		if ( this.redNum > 0 and this.greenNum > 0 and this.blueNum > 0) then
			
			score = score + 1
			invocation = Invocation.new ( )
			
			--[[
			score = score + 100
			subLevel = 1
			level = level + 1
			
			this.redNum   = 0
			this.greenNum = 0
			this.blueNum  = 0
			]]--
			
		-- そろってない
		else
			score = score + 1
			subLevel = subLevel + 1
			
		end
		
		
	end

	
	--=========================================================
	-- 更新
	--=========================================================
	this.update = function(self)
		
		local scorePosX = 250
		local scorePosY = 30
		
		local toBeScoredLightList_updated = {}
		for i = 1, #this.toBeScoredLightList, 1 do
			
			local x = this.toBeScoredLightList[i].x
			local y = this.toBeScoredLightList[i].y
			
			this.toBeScoredLightList[i].x = this.toBeScoredLightList[i].x + this.toBeScoredLightList[i].forceX
			this.toBeScoredLightList[i].y = this.toBeScoredLightList[i].y + this.toBeScoredLightList[i].forceY
			
			this.toBeScoredLightList[i].x = utils.Approach ( this.toBeScoredLightList[i].x, scorePosX, 2 ) 
			this.toBeScoredLightList[i].y = utils.Approach ( this.toBeScoredLightList[i].y, scorePosY, 2 ) 
			
			
			this.toBeScoredLightList[i].forceX = this.toBeScoredLightList[i].forceX + utils.getSign(scorePosX - this.toBeScoredLightList[i].x)
			this.toBeScoredLightList[i].forceY = this.toBeScoredLightList[i].forceY + utils.getSign(scorePosY - this.toBeScoredLightList[i].y)
			
			if ( 
				math.abs ( this.toBeScoredLightList[i].x - scorePosX ) < 10 
				and
				math.abs ( this.toBeScoredLightList[i].y - scorePosY ) < 10  ) then
				
				score = score + 10
				
				local worldPosX, worldPosY = camera:toWorld ( this.toBeScoredLightList[i].x, this.toBeScoredLightList[i].y ) 
				effectForScoreGettingNumber ( {value=10, x=worldPosX, y=worldPosY} )
				
			else
				
				toBeScoredLightList_updated [ #toBeScoredLightList_updated + 1 ] = this.toBeScoredLightList[i]
				
			end
		end
		this.toBeScoredLightList = toBeScoredLightList_updated
		
	end
	
	
	
	--=========================================================
	-- 描画
	--=========================================================
	this.draw = function(self)
		
		local r, g, b, a = love.graphics.getColor()
		
		local originX = 25
		local originY = 60
		
		this.redPosX   = originX + 10 +   0; this.redPosY   = originY + 10
		this.greenPosX = originX + 10 +  64; this.greenPosY = originY + 10
		this.bluePosX  = originX + 10 + 128; this.bluePosY  = originY + 10
		
		love.graphics.draw ( lightSheet, this.lightRed,   this.redPosX,   this.redPosY,   0, 1,1, 32,32)
		love.graphics.draw ( lightSheet, this.lightGreen, this.greenPosX, this.greenPosY, 0, 1,1, 32,32)
		love.graphics.draw ( lightSheet, this.lightBlue,  this.bluePosX,  this.bluePosY,  0, 1,1, 32,32)
		
		local textColorNormal    = {r=1,g=1,b=1}
		local textColorAttention = {r=1,g=0,b=0}
		
		local textColotRed   = textColorNormal
		local textColotGreen = textColorNormal
		local textColotBlue  = textColorNormal
		
		if ( this.redNum == 0 and this.greenNum  > 0 and this.blueNum >  0 ) then textColotRed   = textColorAttention end
		if ( this.redNum >  0 and this.greenNum == 0 and this.blueNum >  0 ) then textColotGreen = textColorAttention end
		if ( this.redNum >  0 and this.greenNum  > 0 and this.blueNum == 0 ) then textColotBlue  = textColorAttention end
		
		drawText ( "x"..this.redNum,   originX + 10 +   0 + 10, originY + 0, textColotRed )
		drawText ( "x"..this.greenNum, originX + 10 +  64 + 10, originY + 0, textColotGreen )
		drawText ( "x"..this.blueNum,  originX + 10 + 128 + 10, originY + 0, textColotBlue )



		for i = 1, #this.toBeScoredLightList, 1 do
		
			love.graphics.draw ( lightSheet, 
				this.toBeScoredLightList[i].image,
				this.toBeScoredLightList[i].x, this.toBeScoredLightList[i].y,
				0, 1,1, 32,32)
			
		end

		
		
		
		love.graphics.setColor(r, g, b, a)
	end
	
	
		
	return this
end

