require( "sceneManager" )
local utils = require "utils"

require( "featMonitor_highScore1" )
require( "featMonitor_highScore2" )
require( "featMonitor_highScore3" )
require( "featMonitor_nodamageRGB1" )
require( "featMonitor_nodamageRGB2" )
require( "featMonitor_noLevelup1" )
require( "featMonitor_noLevelup2" )
require( "featMonitor_noParticle1" )
require( "featMonitor_noParticle2" )
require( "featMonitor_fukutsu" )


--local json = require("lib/json4lua/json")

--utils.writeFile( stageDataDir .. "test.txt", "あいうえお" )

screenWidth, screenHeight, windowFlags = love.window.getMode()

fonts = {}
fonts.default   = love.graphics.newFont("PixelMplus12-Regular.ttf", 32) --love.graphics.getFont()
fonts.system    = love.graphics.newFont("PixelMplus12-Regular.ttf", 32)

bgm = {}
bgm.title = love.audio.newSource ( "BGM196-171031-itsudemoissyo-ogg.ogg", "stream" )
bgm.play  = love.audio.newSource ( "BGM201-171031-trainingpiano-ogg.ogg", "stream" )
bgm.title:setLooping(true)
bgm.play:setLooping(true)
bgm.title:setVolume ( 0.0 )
bgm.play:setVolume ( 0.0 )

--voice = {}
--voice.bgmInvocation = love.audio.newSource ( "../voice.wav", "static" )

mousePushed   = false
mouseReleased = false
mouseX = 0
mouseY = 0

windowPos_BeforeFullScreen = nil

systemMessages = {}

featMonitorList = {}
featInfo = {}


buttonBGM = Button.new( fonts.system, "♪")
buttonBGM.width  = 35
buttonBGM.height = 35
buttonBGM.x = screenWidth - 18
buttonBGM.y = screenHeight - 18
buttonBGM.colorStock.normal = {r = 1.0, g = 1.0, b = 1.0, a = 0.2}

shader = {}
shader.grayscale = love.graphics.newShader[[
    vec4 effect( vec4 color, Image texture, vec2 texture_coords, vec2 screen_coords) {
      vec4 pixel = Texel(texture, texture_coords);
      number gray = pixel.r * 0.30 + pixel.g * 0.59 + pixel.b * 0.11;
//    number gray = (pixel.r + pixel.g + pixel.b) / 3.0;
      pixel.r = gray;
      pixel.g = gray;
      pixel.b = gray;
      pixel.a = pixel.a*0.4;

      return pixel;
    }
  ]]
  
--=========================================================
-- 起動時処理
--=========================================================
function love.load()
	
	--love.mouse.setGrabbed(true)
	
	math.randomseed ( os.time() )

	--love.physics.setMeter(64)
	
	sceneManager = SceneManager.new()
	
	sceneManager.sceneList.play:init()
	
	canvas = love.graphics.newCanvas()
	--love.graphics.setBackgroundColor (1,1,1,1)
	
	
	-- 功績情報：ファイルから読み込む
	toLoad()
--[[
	featInfo = {
		highScore1 = false,
		highScore2 = false,
		highScore3 = false,
		nodamageRGB1 = false,
		nodamageRGB2 = false,
		noLevelup1 = false,
		noLevelup2 = false,
		noParticle1 = false,
		noParticle2 = false,
		fukutsu = false,
	}
]]--

	
	featMonitorList [ #featMonitorList + 1 ] = FeatMonitor_highScore1.new()
	featMonitorList [ #featMonitorList + 1 ] = FeatMonitor_highScore2.new()
	featMonitorList [ #featMonitorList + 1 ] = FeatMonitor_highScore3.new()
	featMonitorList [ #featMonitorList + 1 ] = FeatMonitor_nodamageRGB1.new()
	featMonitorList [ #featMonitorList + 1 ] = FeatMonitor_nodamageRGB2.new()
	featMonitorList [ #featMonitorList + 1 ] = FeatMonitor_noLevelup1.new()
	featMonitorList [ #featMonitorList + 1 ] = FeatMonitor_noLevelup2.new()
	featMonitorList [ #featMonitorList + 1 ] = FeatMonitor_noParticle1.new()
	featMonitorList [ #featMonitorList + 1 ] = FeatMonitor_noParticle2.new()
	featMonitorList [ #featMonitorList + 1 ] = FeatMonitor_fukutsu.new()
	
	
end


--=========================================================
-- 終了時処理
--=========================================================
function love.quit()
	
	print("終了イベント")
	
	toSave()
	
	
	return false --trueにすると終了を抑制する
end

--=========================================================
-- 入力操作
--=========================================================
function love.keypressed ( key )

	sceneManager:keypressed ( key )

	if ( key == "f9" ) then isFPSShown = not isFPSShown end
	
	if ( key == "f11" ) then 
		local isFullScreen, fulScreenType = love.window.getFullscreen()
		
		-- フルスクリーンにする前にウィンドウの位置を覚えておく		
		if ( not isFullScreen ) then
			
			windowPos_BeforeFullScreen = {x=0,y=0,d=1,frameCount = 0}
			windowPos_BeforeFullScreen.x, windowPos_BeforeFullScreen.y, windowPos_BeforeFullScreen.d
			= love.window.getPosition()
		
		end
		
		love.window.setFullscreen( not isFullScreen, fstype )
		
	end
	
end

function love.mousepressed(x, y, button, istouch)

	love.mouse.setGrabbed(true)
	sceneManager:mousepressed ( x, y, button )
	mousePushed = true

end

function love.mousereleased(x, y, button, istouch)

	love.mouse.setGrabbed(false)
	sceneManager:mousereleased ( x, y, button )
	mouseReleased = true

end

function love.mousemoved(x, y, dx, dy, istouch)
	
	sceneManager:mousemoved ( x, y, dx, dy, istouch)
	--text = "mousemoved: (" .. x .. ", " .. y .. ")-("..dx..","..dy..")"
	--print(text)

end

function love.wheelmoved( x, y )
	
	sceneManager:wheelmoved ( x, y )
	
end


function addSystemMessage ( message ) 
	
	systemMessages [ #systemMessages + 1 ] = {
		count = 100,
		message = message
	}
	
end

--=========================================================
-- 更新
--=========================================================
function love.update(dt)
	
	
	sceneManager:update(dt)
	
	mouseReleased = false
	mousePushed   = false
	
	mouseX, mouseY = love.mouse.getPosition()
	
	
	for i = 1, #systemMessages, 1 do
		systemMessages[i].count = systemMessages[i].count - 1
	end
	local isMessageDelete = true
	while (isMessageDelete) do isMessageDelete = false
		
		if ( #systemMessages > 0 ) then
			for i = 1, #systemMessages, 1 do
				if ( systemMessages[i].count <= 0 ) then
					table.remove ( systemMessages, i )
					isMessageDelete = true
					break
				end
			end
		end
		
	end
	
	-- ウィンドウモードに戻したとき、以前の位置にする
	local isFullScreen, fulScreenType = love.window.getFullscreen()
	if ( not isFullScreen and windowPos_BeforeFullScreen ~= nil) then
		
		windowPos_BeforeFullScreen.frameCount = windowPos_BeforeFullScreen.frameCount + 1
		
		-- なぜか少し待ってから実行しないとうまくいかない
		if ( windowPos_BeforeFullScreen.frameCount > 30 ) then

			addSystemMessage ("ウィンドウモードに戻しました")
		
			love.window.setPosition (
				windowPos_BeforeFullScreen.x, windowPos_BeforeFullScreen.y, windowPos_BeforeFullScreen.d
			)
			windowPos_BeforeFullScreen = nil
		end
	end

end

--=========================================================
-- 描画
--=========================================================
function love.draw()
	
	sceneManager:draw()
	
	--love.graphics.setCanvas(canvas)
	love.graphics.setCanvas()
	love.graphics.draw ( sceneManager.canvas )
	


	love.graphics.setCanvas()
	love.graphics.draw ( canvas, screenWidth/2, screenHeight/2, math.rad(0), 1.0,1.0, screenWidth/2, screenHeight/2 )
	
	local fontSystem = fonts.system
	
	local messageY = 0
	for i = 1, #systemMessages, 1 do
		
		love.graphics.setFont(fontSystem)
		love.graphics.setColor(0,0,0,1)
		
		love.graphics.print ( systemMessages[i].message, 5, 5 + messageY, math.rad(0), 1,1,0,0)
		love.graphics.setColor(1,1,1,1)
		love.graphics.setFont(fonts.default)

		messageY = messageY + fonts.system:getHeight(systemMessages[i].message) + 5
		
	end	
	if ( isFPSShown ) then drawFPS() end
	
	
end

local isFPSShown = false
function drawFPS()

	local currentFont = love.graphics.getFont()
	
	local fps = string.format("%02d", love.timer.getFPS() )
	local fontSystem = fonts.system
	
	local fpsMessage = "FPS : "..fps
	
	love.graphics.setFont(fontSystem)
	love.graphics.setColor(1,1,0,1)
	love.graphics.print ( fpsMessage, screenWidth - fonts.system:getWidth(fpsMessage)-5, 5, math.rad(0), 1,1,0,0)
	love.graphics.setColor(1,1,1,1)
	love.graphics.setFont(currentFont)

end


function math.sign(x)
   if x<0 then
     return -1
   elseif x>0 then
     return 1
   else
     return 0
   end
end


--=========================================================
-- セーブデータ書き込み
--=========================================================
toSave = function () 

	-- セーブデータ書き込み
	local saveData = {}
	for i = 1, 1100, 1 do
		
		local data = math.random ( 0,9999 )
		
		saveData [ #saveData + 1 ] = data
		
	end
	
--[[
	featInfo = {
		highScore1 = false,
		highScore2 = false,
		highScore3 = false,
		nodamageRGB1 = true,
		nodamageRGB2 = false,
		noLevelup1 = false,
		noLevelup2 = false,
		noParticle1 = false,
		noParticle2 = false,
		fukutsu = false,
	}
]]--
	local trueFlag = 32

	saveData [ 10  ] = ternary ( featInfo [ "highScore1"   ], trueFlag, math.random(100, 999) )
	saveData [ 100 ] = ternary ( featInfo [ "highScore2"   ], trueFlag, math.random(100, 999) )
	saveData [ 200 ] = ternary ( featInfo [ "highScore3"   ], trueFlag, math.random(100, 999) )
	saveData [ 300 ] = ternary ( featInfo [ "nodamageRGB1" ], trueFlag, math.random(100, 999) )
	saveData [ 400 ] = ternary ( featInfo [ "nodamageRGB2" ], trueFlag, math.random(100, 999) )
	saveData [ 500 ] = ternary ( featInfo [ "noLevelup1"   ], trueFlag, math.random(100, 999) )
	saveData [ 600 ] = ternary ( featInfo [ "noLevelup2"   ], trueFlag, math.random(100, 999) )
	saveData [ 700 ] = ternary ( featInfo [ "noParticle1"  ], trueFlag, math.random(100, 999) )
	saveData [ 800 ] = ternary ( featInfo [ "noParticle2"  ], trueFlag, math.random(100, 999) )
	saveData [ 900 ] = ternary ( featInfo [ "fukutsu"      ], trueFlag, math.random(100, 999) )
	saveData [ 1000] = ternary ( bgm.title:getVolume() > 0  , trueFlag, math.random(100, 999) )
	
	
	
	local saveDataSerial = ""
	
	for i = 1, #saveData, 1 do
		
		saveDataSerial = saveDataSerial..saveData[i]
		
		if ( i < #saveData ) then
			saveDataSerial = saveDataSerial.."zun"
		end
		
	end
	
	utils.writeFile ( "save.zun", saveDataSerial )
	
	--local data = utils.readFile ( "save.zun" )
	--print(data)
	
end


--=========================================================
-- セーブデータ読み込み
--=========================================================
toLoad = function () 
	
	local saveDataSerial = utils.readFile ( "save.zun" )
	
	local saveData = utils.split ( saveDataSerial, "zun")
	
	local trueFlag = "32"
	
	local isBGM = false
	
	if ( #saveData == 1100 ) then
		
		-- 称号
		featInfo [ "highScore1"   ] = (saveData [ 10  ] == trueFlag)
		featInfo [ "highScore2"   ] = (saveData [ 100 ] == trueFlag)
		featInfo [ "highScore3"   ] = (saveData [ 200 ] == trueFlag)
		featInfo [ "nodamageRGB1" ] = (saveData [ 300 ] == trueFlag)
		featInfo [ "nodamageRGB2" ] = (saveData [ 400 ] == trueFlag)
		featInfo [ "noLevelup1"   ] = (saveData [ 500 ] == trueFlag)
		featInfo [ "noLevelup2"   ] = (saveData [ 600 ] == trueFlag)
		featInfo [ "noParticle1"  ] = (saveData [ 700 ] == trueFlag)
		featInfo [ "noParticle2"  ] = (saveData [ 800 ] == trueFlag)
		featInfo [ "fukutsu"      ] = (saveData [ 900 ] == trueFlag)
		
		isBGM = (saveData [ 1000 ] == trueFlag)
		
	else
		
		if ( saveDataSerial == "zun" ) then
			print ("工場出荷状態："..saveDataSerial)
		else
			print ("セーブデータが存在しないか壊れています。"..#saveData)
		end
		
		isBGM = true
	end


	-- BGM
	if ( isBGM ) then
		bgm.title:setVolume(1.0)
		bgm.play:setVolume(1.0)
		buttonBGM.textOverlapped = ""
	else
		bgm.title:setVolume(0.0)
		bgm.play:setVolume(0.0)
		buttonBGM.textOverlapped = "／"
	end
	

end



