local gamera = require 'gamera'

require( "sub1" )
require( "elu" )
require( "zunda" )
require( "light" )
require( "lightBox" )
require( "invocation" )
require( "particleFromElu" )
require( "gauge" )
require( "resultWindow" )

WINDOW_WIDTH = nil
WINDOW_HEIGHT = nil
WINDOW_FLAGS = nil

frameCount = 0;

eluList = {}
lightList = {}
particleList = {}
zundaList = {}

scoreGettingNumberEffectList = {}
damageGettingNumberEffectList = {}

map = {w = 32 * 50, h = 32 * 40}
--local map = {w = 640, h = 480}
--local player = {x = 0, y = 0, w = 40, h = 40, s = 80}

score = 0
level = 1
subLevel =1

hpMax = 100
hp = hpMax
damageCount = 0

destroyBodyReserveList = {}

invocation = nil
resultWindow = nil

gauge = Gauge.new()


scorePosDefault = {x = 20, y = 10}
scorePos = {}

levelPosDefault = {x = 20, y = 100}
levelPos = {}


skyColor = {}
skyColor[1] = {0.5, 0.5, 1.0} --day1
skyColor[2] = {0.2, 0.0, 1.0} --day2
skyColor[3] = {1.0, 0.4, 0.2} --dusk
skyColor[4] = {0.0, 0.0, 0.4} --night1
skyColor[5] = {0.0, 0.0, 0.0} --night2

skyColor_current = {}
skyColor_goal = {}



PlayScene = {}
PlayScene.new = function ()
	
	local this = {}
	
	this.id = "playScene"
	this.canvas = love.graphics.newCanvas()
	

	--=========================================================
	-- ロード
	--=========================================================
	this.init = function()
	
		WINDOW_WIDTH, WINDOW_HEIGHT, WINDOW_FLAGS = love.window.getMode( )
	
		font       = love.graphics.newFont("PixelMplus12-Regular.ttf", 32)
		fontBig    = love.graphics.newFont("PixelMplus12-Regular.ttf", 100)
		fontSmall  = love.graphics.newFont("PixelMplus12-Regular.ttf", 16)
		fontMiddle = love.graphics.newFont("PixelMplus12-Regular.ttf", 64)
		love.graphics.setFont(font)
	
		camera = gamera.new(0, 0, map.w, map.h)
	
		cloudCanvas       = love.graphics.newCanvas ( map.w, map.h )
		backGroundCanvas  = love.graphics.newCanvas ( map.w, map.h )
		informationCanvas = love.graphics.newCanvas ( WINDOW_WIDTH, WINDOW_HEIGHT )
	
		wall  = love.graphics.newImage ( "wall.png" )
		ceil  = love.graphics.newImage ( "mame.png" )
		zunda = love.graphics.newImage ( "mochi_outlined.png" )
		alert = love.graphics.newImage ( "alert.png" )
		cloud = love.graphics.newImage ( "cloud.png" )
		cloud_zunda = love.graphics.newImage ( "cloud_zunda.png" )
		lightSheet = love.graphics.newImage ( "pipo-hikarimono011.png" )
		particleSheet = love.graphics.newImage ( "pipo-hikarimono007.png" )
		myChar = love.graphics.newImage ( "mychar_normal.png" )
		myChar_damage = love.graphics.newImage ( "mychar_damage.png" )

		myCharScaleX = -1.0
	
		myCharWidth, myCharHeight = myChar:getDimensions( )
		
	
		lightBox = LightBox.new()
	
	
		love.physics.setMeter(64) --the height of a meter our worlds will be 64px
		world = love.physics.newWorld(0, 9.81*64, true) --create a world for the bodies to exist in with horizontal gravity of 0 and vertical gravity of 9.81
		
		world:setContactFilter ( contactFilter )
		
		--World:setCallbacks( beginContact, endContact, preSolve, postSolve )
		world:setCallbacks( nil, nil, nil, postSolve )
	
		objects = {} -- table to hold all our physical objects


		objects.anchor = nil
	
		--testElu = Elu.new ( world, 200, 10 )
	
	
		setupWall()
	
	
	
		--lightTest = Light.new ( world, 100, 500, "green" )
	
--		for i = 1, 3, 1 do addLight ( "red"   ) end
--		for i = 1, 3, 1 do addLight ( "green" ) end
--		for i = 1, 3, 1 do addLight ( "blue"  ) end
		
		
	
	end	
	
	--=========================================================
	-- シーン開始
	--=========================================================
	this.start = function()
	
		score = 0
		level = 1
		subLevel =1
		hp = hpMax
		damageCount = 0
		frameCount = 0
		
		-- 難易度確認用
		--level = 50
		--subLevel =15
		
		resultWindow = nil
		
		scorePos.x = scorePosDefault.x
		scorePos.y = scorePosDefault.y

		levelPos.x = levelPosDefault.x
		levelPos.y = levelPosDefault.y
		
		cloudList = {}
		for i = 1, 100, 1 do
		
			local cloud = {}
			cloud.x = math.random ( 0, map.w )
			cloud.y = math.random ( 0, map.h )
			cloud.distance = math.floor ( i/10 ) -- math.random ( 1, 10 )
			cloud.isZunda = false

			if ( math.random ( 0, 15 ) == 0 and cloud.distance < 7 ) then
				cloud.isZunda = true
			end
			
			cloudList[#cloudList + 1] = cloud
		
		end
		
		
		-- 自キャラの位置
		--let's create a ball
		objects.myChar = {}
		--local x, y = camera:toWorld ( map.w/2, map.h/2 );
		local x = map.w/2
		local y = map.h/2
		--objects.myChar.body = love.physics.newBody(world, WINDOW_WIDTH/2, WINDOW_HEIGHT/2, "dynamic") --place the body in the center of the world and make it dynamic, so it can move around
		objects.myChar.body = love.physics.newBody(world, x, y, "dynamic") --place the body in the center of the world and make it dynamic, so it can move around
		objects.myChar.shape = love.physics.newCircleShape (20) --the ball's shape has a radius of 20
		objects.myChar.fixture = love.physics.newFixture(objects.myChar.body, objects.myChar.shape, 1) -- Attach fixture to body and give it a density of 1.
		objects.myChar.fixture:setRestitution(0.3) --let the ball bounce
		objects.myChar.body:setUserData({obj = nil, name = "myChar"})
		
		lightBox:init()
		
		scoreGettingNumberEffectList = {}
		damageGettingNumberEffectList = {}
		
		for i = 1, 3, 1 do addLight ( "red"   ) end
		for i = 1, 3, 1 do addLight ( "green" ) end
		for i = 1, 3, 1 do addLight ( "blue"  ) end
		
		
		skyColor_current[1] = skyColor[1][1]
		skyColor_current[2] = skyColor[1][2]
		skyColor_current[3] = skyColor[1][3]

		
		-- 功績モニタリング
		for i = 1, #featMonitorList, 1 do
			featMonitorList[i]:init()
		end	

	end

	--=========================================================
	-- シーン終了
	--=========================================================
	this.beforeEnd = function()
		
		-- エルを片付けたり
		for i = 1, #eluList, 1 do
			eluList[i]:destruct()
		end
		eluList = {}
		
		
		-- 光を片付ける
		for i = 1, #lightList, 1 do
			lightList[i]:destruct()
		end
		lightList = {}

		-- パーティクルを片付ける
		for i = 1, #particleList, 1 do
			particleList[i]:destruct()
		end
		particleList = {}

		-- ずんだ餅を片付ける
		for i = 1, #zundaList, 1 do
			zundaList[i]:destruct()
		end
		zundaList = {}
		
		
		-- 雲も片付けましょうか
		cloudList = {}
		
		-- ロープを消しておく
		releaseRope()
		
		-- 自キャラのオブジェクトを片付ける
		objects.myChar.body:destroy()
		
				
	end
	
    
	--=========================================================
	-- 更新
	--=========================================================
	this.update = function(self, dt)
		
		-- 功績モニタリング
		for i = 1, #featMonitorList, 1 do
			featMonitorList[i]:update()
		end
		gotParticleHappened = false
		
		-- 空の色を変化させる
		transitionBackgroundColor()
		
		if ( invocation ~= nil ) then
		
			local invocationResult = invocation:update()
			
			if ( invocationResult == "done" ) then
				invocation = nil
			end
--[[
		elseif ( resultWindow ~= nil ) then
			
			gauge:update() -- 体力ゲージは更新。最後のトドメの部分が減らずに残ってしまう
			
			local resultResult = resultWindow:update()
			
			if ( resultResult == "done" ) then
				resultWindow = nil
			end
]]--
		else
			this:updatePrivate(dt)
		end
		
	end
	-- 更新処理：ここから
	ropeLength = 0;
	ropeLengthChangeMoment = 0;
	--=========================================================
	-- 更新
	--=========================================================
	this.updatePrivate = function(self, dt)
		
		frameCount = frameCount + 1
		if ( frameCount > 9999 ) then frameCount = 0 end
	
		lightBox:update()
		gauge:update()
		
		damageCount = damageCount - 1
		if ( damageCount < 0 ) then damageCount = 0 end
		
		
		if ( resultWindow ~= nil ) then
			
			local resultResult = resultWindow:update()
			
			if ( resultResult == "done" ) then
				resultWindow = nil
			end
		end
		
		
		-- エルを降らせる判定
		local eluFrequency = 100 - level*2
		
		if ( eluFrequency <= 30 ) then eluFrequency = 30 end
	
		if ( frameCount%eluFrequency == 0 ) then
		
			for i = 1, subLevel, 1 do
				local isAllRange = i > 1
				
				if ( resultWindow == nil ) then
					addElu ( isAllRange )
				end
			end
		end
		
		-- 光
		for i = 1, #lightList, 1 do
			lightList[i]:update()
		end
		
		-- ずんだ餅		
		for i = 1, #zundaList, 1 do
			zundaList[i]:update()
		end
		
		
		-- スコア獲得エフェクト
		local scoreGettingNumberEffectList_updated = {}
		for i = 1, #scoreGettingNumberEffectList, 1 do
		
			scoreGettingNumberEffectList[i].y = scoreGettingNumberEffectList[i].y - 1
			scoreGettingNumberEffectList[i].frameCount = scoreGettingNumberEffectList[i].frameCount + 1
			
			if ( scoreGettingNumberEffectList[i].frameCount < 50 ) then
				scoreGettingNumberEffectList_updated [ #scoreGettingNumberEffectList_updated + 1 ] = scoreGettingNumberEffectList[i]
			end
			
		end
		scoreGettingNumberEffectList = scoreGettingNumberEffectList_updated

		-- ダメージの数字エフェクト
		local damageGettingNumberEffectList_updated = {}
		for i = 1, #damageGettingNumberEffectList, 1 do
		
			damageGettingNumberEffectList[i].y = damageGettingNumberEffectList[i].y - 1
			damageGettingNumberEffectList[i].frameCount = damageGettingNumberEffectList[i].frameCount + 1
			
			if ( damageGettingNumberEffectList[i].frameCount < 50 ) then
				damageGettingNumberEffectList_updated [ #damageGettingNumberEffectList_updated + 1 ] = damageGettingNumberEffectList[i]
			end
			
		end
		damageGettingNumberEffectList = damageGettingNumberEffectList_updated
		

		if ( ropeJoint ~= nil ) then
			--[[
			local length = ropeJoint:getMaxLength()
		
			ropeLengthChangeMoment = length

			length = length + ropeLengthChangeMoment;
		
			ropeLengthChangeMoment
		
			if ( length < 50 ) then
				length = 50
			end
			--ropeLengthChangeMoment = ropeLengthChangeMoment + (ropeLengthChangeMoment * -0.3)
		
			ropeJoint:setMaxLength ( length )
			]]--
		
		end
	
		--print("w:"..w)
		
		world:update(dt) --this puts the world into motion
	
		local myCharMoveForceX, myCharMoveForceY = objects.myChar.body:getLinearVelocity()
		if ( myCharMoveForceX > 0 ) then myCharScaleX = -1.0 end
		if ( myCharMoveForceX < 0 ) then myCharScaleX =  1.0 end
	
	
		if love.keyboard.isDown ("right" ) then
--[[		
			recoverToMyChar(9)
			lightBox.redNum = lightBox.redNum + 1
]]--
		elseif love.keyboard.isDown ("left" ) then
--[[		
			damageToMyChar(9)

]]--
		elseif love.keyboard.isDown ("space" ) then
--[[		
			--playFeatAchievedFanfare()
			invocation = Invocation.new()
]]--

		elseif love.keyboard.isDown ("return" ) then
			
--[[		
			for n = 1, 100, 1 do
				addParticle ( math.random(90,110), 120, 100, 100, {r=1,g=1,b=1} ) 
			end
			print(#particleList)
			
			addElu (true)
			print ( #eluList )
			
			--invocation = Invocation.new()
			score = score + 100
			
			--addZunda()
]]--
		end
	
		--camera:setPosition ( objects.myChar.body:getX() + myCharWidth*0.2/2, 300)
		--camera:setPosition ( objects.myChar.body:getX() + myCharWidth*0.2/2, objects.myChar.body:getY() + myCharHeight*0.2/2)
		camera:setPosition ( objects.myChar.body:getX(), objects.myChar.body:getY() + myCharHeight*0.2/2)
	
		--objects.ball.body:getY() + myCharHeight/2)
	
	
		--testElu:update()
	
		for i = 1, #eluList, 1 do
			eluList[i]:update()
		end
		
		local particleList_updated = {}
		for i = 1, #particleList, 1 do
			particleList[i]:update()
			
			if ( particleList[i].isTimeouted == false) then
				particleList_updated [ #particleList_updated + 1 ] = particleList[i]
			else
				particleList[i]:destruct()
			end
		end
		particleList = particleList_updated

		
		for i = 1, #destroyBodyReserveList, 1 do
			if ( destroyBodyReserveList[i] ~= nil ) then
			
				if ( destroyBodyReserveList[i]:isDestroyed() == false ) then
					destroyBodyReserveList[i]:destroy()
				end
			end
		end
		destroyBodyReserveList = {}
		

	end
	
	--=========================================================
	-- 描画
	--=========================================================
	this.draw = function(self)
		
		local r,g,b,a = love.graphics.getColor()
		local currentCanvas = love.graphics.getCanvas()
		love.graphics.setCanvas ( self.canvas )

		love.graphics.clear(1,1,1,1)
		
		
		
		--self.button:draw()
		
		self:drawPrivate()
		
		
		love.graphics.setCanvas ( currentCanvas )
		love.graphics.setColor(r,g,b,a)
	end
	--=========================================================
	-- 描画処理：ここから
	--=========================================================
	this.drawPrivate = function(self)
	
		--love.graphics.setCanvas ()
		--love.graphics.clear(1,1,1,1)


		--love.graphics.clear(0.5,0.5,1.0,1)
	
		-- 雲キャンバス
		love.graphics.setCanvas (cloudCanvas)
		
			--love.graphics.clear(0.0,0.0,0.4,1,0) -- 夜
			--love.graphics.clear(0.0,0.0,0.0,1,0) -- 夜2
			--love.graphics.clear(1.0,0.4,0.2,1,0) -- 夕方
			--love.graphics.clear(0.5,0.5,1.0,1,0) -- 昼1
			--love.graphics.clear(0.2,0.0,1.0,1,0) -- 昼2

			love.graphics.clear(skyColor_current)
		
			local cameraX, cameraY = camera:getPosition()
			local mapCenterX = map.w/2
			local mapCenterY = map.h/2
		
		
			for i = 1, #cloudList, 1 do
			

				local cloudDistanceX = (mapCenterX- cameraX) / (cloudList[i].distance + 1)
				local cloudDistanceY = (mapCenterY- cameraY) / (cloudList[i].distance + 1)
			
				local alpha = cloudList[i].distance / 10

				love.graphics.setColor( 1.0, 1.0, 1.0, alpha )

				local scale = ( (cloudList[i].distance + 10)/2 ) / 10
				
				local cloudImage = cloud
				
				if ( cloudList[i].isZunda ) then
					cloudImage = cloud_zunda
				end
				
				love.graphics.draw ( cloudImage, 
					cloudList[i].x - cloudDistanceX
					, 
					cloudList[i].y - cloudDistanceY
					,
					0,
					scale, scale
				)
	
					
			end
		
		love.graphics.setCanvas ()
	
		-- 背景キャンバス（壁とか床とか）
		love.graphics.setCanvas (backGroundCanvas)
		
			love.graphics.clear(0,0,0,0)
		
			love.graphics.setColor(1,1,1,1)
		
			displayWall()
		
			--love.graphics.setColor( 0.7, 1.0, 0.7 )
			--love.graphics.rectangle ( "fill", WINDOW_WIDTH/2, WINDOW_HEIGHT/2, 50, 200 )


			--love.graphics.setColor( 0.0, 0.0, 1.0 )
			--love.graphics.polygon ( "fill", objects.ground.body:getWorldPoints ( objects.ground.shape:getPoints() ) )

			--love.graphics.setColor( 0.0, 1.0, 0.0 )
			--love.graphics.polygon ( "fill", objects.block1.body:getWorldPoints ( objects.block1.shape:getPoints() ) )
			--love.graphics.polygon ( "fill", objects.block2.body:getWorldPoints ( objects.block2.shape:getPoints() ) )

			--love.graphics.setColor( 1.0, 0.0, 0.0 )
			--love.graphics.circle( "line", objects.myChar.body:getX(), objects.myChar.body:getY(), objects.myChar.shape:getRadius() )
		
			love.graphics.setColor( 1.0, 1.0, 1.0 )
		
		love.graphics.setCanvas ()
	
	
		--love.graphics.draw(backGroundCanvas, 0, 0)
	
		-- カメラでの描画
		camera:draw(function(l, t, w, h)

			love.graphics.draw(cloudCanvas, 0, 0)
			love.graphics.draw(backGroundCanvas, 0, 0)

			--エル
			for i = 1, #eluList, 1 do
				eluList[i]:draw()
			end
			
			-- エルが消滅した後のパーティクル
			for i = 1, #particleList, 1 do
				particleList[i]:draw()
			end
		
			-- 光
			for i = 1, #lightList, 1 do
				lightList[i]:draw()
			end
			
			-- ずんだ餅		
			for i = 1, #zundaList, 1 do
				zundaList[i]:draw()
			end
		
			drawRope()
			
			-- ロープのアンカーの当たり判定
			if ( objects.anchor ~= nil ) then
				--love.graphics.setColor( 1.0, 0.0, 1.0 )
				--love.graphics.circle( "line", objects.anchor.body:getX(), objects.anchor.body:getY(), objects.anchor.shape:getRadius() )
			end

			-- スコア獲得エフェクト
			for i = 1, #scoreGettingNumberEffectList, 1 do
				
				local text = tostring(scoreGettingNumberEffectList[i].score)
				if ( scoreGettingNumberEffectList[i].score >= 0 ) then
					text = "+"..text
				end
				drawText (
					text,
					scoreGettingNumberEffectList[i].x,
					scoreGettingNumberEffectList[i].y,
					scoreGettingNumberEffectList[i].color,
					scoreGettingNumberEffectList[i].font
				)
			end

			-- ダメージ数エフェクト
			for i = 1, #damageGettingNumberEffectList, 1 do
				
				local text = tostring(damageGettingNumberEffectList[i].damageValue)
				if ( damageGettingNumberEffectList[i].damageValue >= 0 ) then
					text = "-"..text
				end
				drawText (
					text,
					damageGettingNumberEffectList[i].x,
					damageGettingNumberEffectList[i].y,
					damageGettingNumberEffectList[i].color,
					damageGettingNumberEffectList[i].font
				)
			end
			
			-- 自キャラ
			--displaySprite ( myChar, WINDOW_WIDTH/2, WINDOW_HEIGHT/2 )
	
			local x = objects.myChar.body:getX()
			local y = objects.myChar.body:getY()
			local rotate = 0
			local scaleX = myCharScaleX * 0.2
			local scaleY = 0.2
			local originX = myCharWidth/2
			local originY = myCharHeight/2
			local kx = 0
			local ky = 0
			
			local myCharImage = myChar
			local myCharDrawColor = {r = 1, g = 1, b = 1, a = 1}
			if ( damageCount > 0 ) then
				myCharImage = myChar_damage
				
				if (frameCount%2 == 0) then myCharDrawColor.a = 0 end
			end

			love.graphics.setColor( myCharDrawColor.r, myCharDrawColor.g, myCharDrawColor.b, myCharDrawColor.a )
			
			love.graphics.draw( myCharImage, 
				x, y, -- 位置
				rotate,    -- 回転
				scaleX, scaleY, -- スケール
				originX, originY, -- オリジン
				kx, ky  -- Shearing factor
			)
	
			-- love.graphics.draw ( myCharBase, meChar_right,  WINDOW_WIDTH/2 - w/2, WINDOW_HEIGHT/2 - h/2 )
			
			love.graphics.setColor (1,1,1,1)
		end)

		if ( invocation ~=nil ) then
			invocation:draw()
		end
		
		-- 情報キャンバス
		love.graphics.setCanvas ( informationCanvas )
		
			love.graphics.clear(0,0,0,0)
			
			-- 結果パネル
			if ( resultWindow ~= nil ) then resultWindow:draw() end
			
			-- 体力ゲージ
			gauge:draw()
		
			-- 集めた光		
			lightBox:draw()
		
			-- スコア
			local scoreInfo = tostring(score)
			while(string.len(scoreInfo) <= 10)
			do
				scoreInfo = "0"..scoreInfo
			end
			--drawText ( "SCORE "..scoreInfo, 20, 10  );
			drawText ( "SCORE "..scoreInfo, scorePos.x, scorePos.y  );

			-- レベル
			local levelInfo = "LEVEL "..tostring(level).."-"..tostring(subLevel)
			--drawText ( levelInfo, 20, 100  );
			drawText ( levelInfo, levelPos.x, levelPos.y  );
			
			-- 功績モニタリング（何か表示するものがあれば）
			for i = 1, #featMonitorList, 1 do
				featMonitorList[i]:draw()
			end
			
		
		love.graphics.setCanvas ()

		love.graphics.draw(informationCanvas, 0, 0)

	
--[[	
		love.graphics.setBlendMode("add")
		
		love.graphics.setColor(1, 0, 0, 1)
		love.graphics.circle ("fill", screenWidth/2 - 50, screenHeight/2+100, 100)

		love.graphics.setColor(0, 1, 0, 1)
		love.graphics.circle ("fill", screenWidth/2 + 50, screenHeight/2+100, 100)	

		love.graphics.setBlendMode("alpha")
]]--	
	end



	--=========================================================
	-- マウス受付（押した）
	--=========================================================
	this.mousepressed = function ( self, x, y, button, istouch )
		
		if ( resultWindow ~= nil ) then return end
		
		-- 左クリック
		if ( button == 1 ) then
			if ( damageCount == 0 ) then
				setRope ( x, y )
			end
		end
	end
	--=========================================================
	-- マウス受付（離した）
	--=========================================================
	this.mousereleased = function (self, x, y, button, istouch)
	
		if ( resultWindow ~= nil ) then return end

		-- 左クリック
		if ( button == 1 ) then
			releaseRope ()
		end
	end



	--=========================================================
	-- 壁セット
	--=========================================================
	function setupWall () 
	
		local wallBlockWidth, wallBlockHeight = wall:getDimensions( )
	
		wallList = {}
		
		-- 床
		for i = 1, map.w, wallBlockWidth do
		
			local x, y = camera:toWorld ( i + wallBlockWidth/2, map.h - wallBlockHeight/2  );
		
			local block = {}
			block.body = love.physics.newBody(world, x, y, "static")
			block.shape = love.physics.newRectangleShape(wallBlockWidth, wallBlockHeight)
			block.fixture = love.physics.newFixture(block.body, block.shape)
			block.fixture:setRestitution(0.3) --let the ball bounce

			block.body:setUserData ( {obj=nil, name="ground", image=wall})
		
			wallList[#wallList + 1] = block

		
		end

		-- 天井
		for i = 1, map.w, wallBlockWidth do
		
			local x, y = camera:toWorld ( i + wallBlockWidth/2, wallBlockHeight/2 );
		
			local block = {}
			block.body = love.physics.newBody(world, x, y, "static")
			block.shape = love.physics.newRectangleShape(wallBlockWidth, wallBlockHeight)
			block.fixture = love.physics.newFixture(block.body, block.shape)
			block.fixture:setRestitution(0.3) --let the ball bounce
			
			block.body:setUserData ( {obj=nil, name="ceil", image=ceil} )
		
			wallList[#wallList + 1] = block
		
		end

		-- 壁
		for i = 1, map.h, wallBlockHeight do
		
			local x, y = camera:toWorld ( wallBlockWidth/2, i + wallBlockHeight/2  );
		
			local block = {}
			block.body = love.physics.newBody(world, x, y, "static")
			block.shape = love.physics.newRectangleShape(wallBlockWidth, wallBlockHeight)
			block.fixture = love.physics.newFixture(block.body, block.shape)
			block.fixture:setRestitution(0.3) --let the ball bounce

			block.body:setUserData ( {obj=nil, name="wall", image=wall})
		
			wallList[#wallList + 1] = block
		
		end
	
		
		-- 壁
		for i = 1, map.h, wallBlockHeight do
		
			local x, y = camera:toWorld ( map.w -wallBlockWidth/2, i + wallBlockHeight/2  );
		
			local block = {}
			block.body = love.physics.newBody(world, x, y, "static")
			block.shape = love.physics.newRectangleShape(wallBlockWidth, wallBlockHeight)
			block.fixture = love.physics.newFixture(block.body, block.shape)
			block.fixture:setRestitution(0.3) --let the ball bounce

			block.body:setUserData ( {obj=nil, name="wall", image=wall})
		
			wallList[#wallList + 1] = block
		
		end

	
	end

	--=========================================================
	-- 壁描画
	--=========================================================
	function displayWall () 

		for i = 1, #wallList, 1 do

			local x = wallList[i].body:getX()
			local y = wallList[i].body:getY()
			local rotate = 0
			local scaleX = 1.0
			local scaleY = 1.0
			local originX = 16
			local originY = 16
			local kx = 0
			local ky = 0
			
			local image = wallList[i].body:getUserData().image
			
			love.graphics.draw( image, 
				x, y, -- 位置
				rotate,    -- 回転
				scaleX, scaleY, -- スケール
				originX, originY, -- オリジン
				kx, ky  -- Shearing factor
			)
		end

	end


	function displaySprite ( imageObj, x, y )
	
		local w,h = myChar:getDimensions( )
	
		love.graphics.draw ( imageObj, x - w/2, y - h/2 )
	
	end

	
	
	--=========================================================
	-- シーン開始
	--=========================================================
	this.enter = function(self)
		print("開始："..self.id)
		
		love.audio.play ( bgm.play )
		
		this:start()
			
	end
	--=========================================================
	-- シーン終了
	--=========================================================
	this.leave = function(self)
		print("終了："..self.id)

		this:beforeEnd()

		love.audio.stop ( bgm.play )		
	end
	
	return this
end






















