
require( "titleScene" )
require( "playScene" )


SceneManager = {}
SceneManager.new = function ()
	
	local this = {}
	
	this.sceneList = {
		title = TitleScene.new(),
--		stageSelect = StageSelectScene.new(),
		play  = PlayScene.new()
	}
	
	this.currentScene   = "title"
	this.nextScene      = ""
	this.previousScene  = ""
	
	this.sceneList[this.currentScene]:enter()
	
	this.sceneTransitionShaderProgress = 0
	
	this.canvas = love.graphics.newCanvas()
	
	this.shader = nil
	
	this.shaderPixel = love.graphics.newShader[[
		
		extern number fadeThickness;
		
		number pixel_amount = fadeThickness * 10;
		
		vec4 effect(vec4 color, Image tex, vec2 p, vec2 pc) {
			vec4 finTex = Texel(tex, p);
			vec2 np;
			float a, b, c;
			
			vec2 p1 = floor(p*(100/pixel_amount))/(100/pixel_amount);
			vec2 p2 = floor(p*(100/-pixel_amount))/(100/-pixel_amount);
			vec4 finTex2 = Texel(tex, p1);
			vec4 finTex3 = Texel(tex, p2);

			vec4 finTex4 = mix(finTex2, finTex3, 0.5);
			
			return finTex4;
		}
	
	]]

	this.shaderPinch = love.graphics.newShader[[
		
		extern number fadeThickness;
		
		vec2 pinch_pos = vec2(fadeThickness * 1, fadeThickness * 1);
		
		vec4 effect(vec4 color, Image tex, vec2 p, vec2 pc) {
			vec4 finTex = Texel(tex, p);
			vec2 np;
			float a, b, c;

			np = vec2(p.x - pinch_pos.x, p.y - pinch_pos.y);

			a = length(np);
			b = atan(np.y, np.x);
			a = sqrt(a);
			np = a * vec2(cos(b), sin(b));
			p = np + pinch_pos;
			finTex = Texel(tex, p);
			
			return finTex;
		}
	
	]]



	this.shaderFade = love.graphics.newShader[[
		extern number size;

		extern number fadeThickness;
		
		vec4 effect(vec4 color, Image texture, vec2 texture_coords, vec2 screen_coords) {
			
			vec2 texture_coordsBase;
			texture_coordsBase.x = texture_coords.x - mod(texture_coords.x, size);
			texture_coordsBase.y = texture_coords.y - mod(texture_coords.y, size);
			
			vec4 pixelBase = Texel(texture, texture_coordsBase);
			vec4 pixel     = Texel(texture, texture_coords);

			vec4 fadeColor = vec4(1,1,1,1);
			
			vec4 returnColor = mix(pixel, fadeColor, fadeThickness);
			
			return returnColor;
		}
	]]

		
	--=========================================================
	-- 入力
	--=========================================================
	this.keypressed = function( self,  key )
		--print (key)
		
		--if (key == "f1") then self.nextScene = "title" end
		--if (key == "f2") then self.nextScene = "play" end

		if ( self.sceneList [ self.currentScene ].keypressed ~= nil ) then
			self.sceneList [ self.currentScene ]:keypressed ( key )
		end
	end
	
	this.mousepressed = function ( self, x, y, button, istouch)
	
		if ( self.sceneList [ self.currentScene ].mousepressed ~= nil ) then
			self.sceneList [ self.currentScene ]:mousepressed ( x, y, button, istouch )
		end
	end
	
	this.mousereleased = function ( self, x, y, button, istouch)

		if ( self.sceneList [ self.currentScene ].mousereleased ~= nil ) then
			self.sceneList [ self.currentScene ]:mousereleased ( x, y, button, istouch )
		end

	end
	
	this.mousemoved = function ( self, x, y, dx, dy, istouch )

		if ( self.sceneList [ self.currentScene ].mousemoved ~= nil ) then
			self.sceneList [ self.currentScene ]:mousemoved ( x, y, dx, dy, istouch )
		end
		
	end

	this.wheelmoved = function (self, x, y)
		
		if ( self.sceneList [ self.currentScene ].wheelmoved ~= nil ) then
			self.sceneList [ self.currentScene ]:wheelmoved ( x, y )
		end
		
	end
	
	--=========================================================
	-- 更新
	--=========================================================
	this.update = function(self, dt)
		
		buttonBGM:update()
		if ( buttonBGM.isClicked ) then
			
			local currentVolume = bgm.title:getVolume()
			
			if ( currentVolume > 0 ) then

				bgm.title:setVolume(0)
				bgm.play:setVolume(0)
				
				buttonBGM.textOverlapped = "／"
			else

				bgm.title:setVolume(1.0)
				bgm.play:setVolume(1.0)

				buttonBGM.textOverlapped = ""
			end

		end
		
		
		
		if ( self.nextScene ~= "" ) then 

			self.shader = self.shaderPixel
			--self.shader = self.shaderFade
			--self.shader = self.shaderPinch
			
			
			self.sceneTransitionShaderProgress = self.sceneTransitionShaderProgress + 0.04

			if ( self.sceneTransitionShaderProgress > 1.0 ) then 
				self.sceneTransitionShaderProgress = 1.0
				self.previousScene = self.currentScene
				self.currentScene  = self.nextScene
				self.nextScene     = ""
				
				self.sceneList[self.previousScene]:leave()
				self.sceneList[self.currentScene]:enter()
			end
		
		elseif ( self.previousScene ~= "" ) then
			self.sceneTransitionShaderProgress = self.sceneTransitionShaderProgress - 0.04

			if ( self.sceneTransitionShaderProgress < 0.0 ) then 
				self.sceneTransitionShaderProgress = 0.0
				self.previousScene = ""
				
				self.shader = nil
			end
		end
		
		if ( self.nextScene == "" ) then 
			self.sceneList [ self.currentScene ] : update(dt)
		end
		
		if ( self.shader ~= nil ) then
			self.shader:send("fadeThickness", self.sceneTransitionShaderProgress)
		end
		
	end
	
	--=========================================================
	-- 描画
	--=========================================================
	this.draw = function(self)

		love.graphics.setShader(self.shader)
		self.sceneList [ self.currentScene ] : draw()
		love.graphics.setShader()
		
		buttonBGM:draw()
		
--[[		
		-- 現在のシーンを描画（シーン側のキャンバスに）
		self.sceneList [ self.currentScene ] : draw()
		
		-- シーンマネージャーのキャンバスに変更
		local currentCanvas = love.graphics.getCanvas()
		love.graphics.setCanvas(self.canvas)
		love.graphics.clear(0,0,0,1)
		
		-- 現在のシーンのキャンバスを、マネージャーのキャンバスに描画
		love.graphics.setShader(self.shader)
		love.graphics.draw ( self.sceneList [ self.currentScene ].canvas, 0, 0 )
		love.graphics.setShader()
		
		-- キャンバスを戻す
		love.graphics.setCanvas(currentCanvas)

		--love.graphics.setCanvas()
		
		--love.graphics.setShader(self.shader)

		--love.graphics.draw ( self.canvas )

		--love.graphics.setShader()
]]--
	
	end
	
	
	return this
end
