local utils = require "utils"

--=========================================================
-- 三項演算子
--=========================================================
function ternary ( condition, a, b)
	
	local result = a
	
	if ( condition ) then 
		result = a 
	else
		result = b
	end
	
	return result
	
end

--=========================================================
-- 文字を描画
--=========================================================
function drawText ( text, x, y, givenColor, font)
	
	local r,g,b = love.graphics.getColor();
	
	-- フォントを設定
	local currentFont = love.graphics.getFont()
	if (font == nil) then font = currentFont end
	love.graphics.setFont(font)
	
	
	-- 影を描画
	love.graphics.setColor(0,0,0)
	love.graphics.print(text, x + 1, y + 1)
	
	local color = givenColor
	if ( color == nil ) then
		color = {}
		color.r = 1
		color.g = 1
		color.b = 1
	end
	
	love.graphics.setColor ( color.r, color.g, color.b)
	
	-- 本体を描画	
	love.graphics.print(text, x, y)	
	
	-- フォントを戻す
	love.graphics.setFont(currentFont)
	
	
	love.graphics.setColor(r,g,b);
end

--=========================================================
-- 称号達成時のファンファーレ
--=========================================================
function playFeatAchievedFanfare ()

	local s = love.audio.newSource("fanfare09.mp3", "static")
	love.audio.play(s)
	
	
	toSave()
	
end

--=========================================================
-- エルを出現させる
--=========================================================
function addElu ( isAllRange )
	
	--- マップ全体が範囲
	if ( isAllRange ) then
	
		local cameraX, cameraY = camera:getPosition()
	
		local newEluX = math.random ( 0, map.w	)
	
		local newEluY = -50
	
		newElu = Elu.new (world, newEluX, newEluY )
		eluList [#eluList + 1] = newElu
	
	--- 映っている範囲内で出現	
	else
		local cameraX, cameraY = camera:getPosition()
	
		local newEluX = math.random ( 
			cameraX - WINDOW_WIDTH/2,  
			cameraX + WINDOW_WIDTH/2
		)
	
		local newEluY = -50
	
		--math.random ( 
		--	-100,  
		--	-10
		--)
	
		newElu = Elu.new (world, newEluX, newEluY )
		eluList [#eluList + 1] = newElu
	end
	
	
	-- エルの数が多くなりすぎないように上限を設けておく
	if ( #eluList > 200 ) then
		
		eluList[1]:destruct()
		deleteElu ( eluList[1] )
		
	end
	

end


--=========================================================
-- ロープを設置
--=========================================================
function setRope ( anchorX, anchorY )

	--print ( anchorX..","..anchorY )
	
	-- 既存のが残っていれば破棄する
	if ( ropeJoint ~= nil ) then
		releaseRope()
	end
	
	local worldX, worldY = camera:toWorld ( anchorX, anchorY )
	
	
	local distance = ( (objects.myChar.body:getX() - worldX)^2 + (objects.myChar.body:getY() - worldY)^2 )^0.5
	
	
	objects.myChar.body:applyForce (
		(worldX - objects.myChar.body:getX()) * 64,
		(worldY - objects.myChar.body:getY()) * 64
	)
	


	-- アンカー
	objects.anchor = {}
	objects.anchor.body = love.physics.newBody(world, 
		worldX, 
		worldY,
	"static")
	objects.anchor.shape = love.physics.newCircleShape(10) 
	objects.anchor.fixture = love.physics.newFixture(objects.anchor.body, objects.anchor.shape)
	objects.anchor.fixture:setRestitution(0.0)
	
	objects.anchor.body:setUserData( {obj=nil, name="ropeAnchor"});

	

--[[
	-- ロープ(newPrismaticJoint)
	ropeJoint = love.physics.newPrismaticJoint( -- body1, body2, x1, y1, x2, y2, collideConnected )
		objects.myChar.body, objects.anchor.body,
		objects.myChar.body:getX(), objects.myChar.body:getY(),
		objects.anchor.body:getX(), objects.anchor.body:getY(),
		0.1,0,
		false
	)
]]--
--[[
	-- ロープ(Weld)
	ropeJoint = love.physics.newWeldJoint( -- body1, body2, x1, y1, x2, y2, collideConnected )
		objects.myChar.body, objects.anchor.body,
		objects.myChar.body:getX(), objects.myChar.body:getY(),
		objects.anchor.body:getX(), objects.anchor.body:getY(),
		false
	)
]]--


	-- ロープ
	ropeJoint = love.physics.newRopeJoint( --body1, body2, x1, y1, x2, y2, maxLength, collideConnected )
		objects.myChar.body, objects.anchor.body,
		objects.myChar.body:getX(), objects.myChar.body:getY(),
		objects.anchor.body:getX(), objects.anchor.body:getY(),
		distance * 1.0,
		false
	)

	

	
	

end

--=========================================================
-- ロープをリリース
--=========================================================
function releaseRope ()
	
	if ( objects.anchor ~= nil ) then
		objects.anchor.body:destroy()
		objects.anchor = nil
		--world:destroyBody()

		ropeJoint = nil
	end

--	if ( ropeJoint ~= nil ) then
--		ropeJoint:destroy()
--		ropeJoint = nil
--	end	
end


--=========================================================
-- ロープを描画
--=========================================================
function drawRope ()
	
	if ( ropeJoint ~= nil ) then
		
		local x1, y1, x2, y2 = ropeJoint:getAnchors()
		
		love.graphics.setColor( 1.0, 0.7, 0.0 )
		
		local lw = love.graphics.getLineWidth()
		
		love.graphics.setLineWidth(4)

		--love.graphics.setLineStyle("rough")
		love.graphics.setLineStyle("smooth")

		love.graphics.line ( ropeJoint:getAnchors() )

		love.graphics.setLineWidth(lw)
		
		
	end	
end

--=========================================================
-- 背景色を昼夕夜と変化させる
--=========================================================
transitionBackgroundColor = function () 
	
--	skyColor.day   = {0.5, 0.5, 1.0}
--	skyColor.dusk  = {1.0, 0.4, 0.2}
--	skyColor.night = {0.0, 0.0, 0.4}
--	skyColor.current = {}	

	for i = 1, #skyColor, 1 do
		
		if ( skyColor_current[1] == skyColor[i][1] and skyColor_current[2] == skyColor[i][2] and skyColor_current[3] == skyColor[i][3] ) then
		
			local nextIndex = i + 1
			if ( nextIndex > #skyColor ) then nextIndex = 1 end
			
			skyColor_goal[1] = skyColor[nextIndex][1]
			skyColor_goal[2] = skyColor[nextIndex][2]
			skyColor_goal[3] = skyColor[nextIndex][3]
		end
	end
	
	-- 目指す色に向けて変化させる
	skyColor_current[1], skyColor_current[2], skyColor_current[3] = utils.Move3d (
		skyColor_current[1], skyColor_current[2], skyColor_current[3],
		skyColor_goal[1], skyColor_goal[2], skyColor_goal[3], 
		0.0003
		--0.05
	)

	

end

--=========================================================
-- エル消滅時のパーティクルを画面に出現させる
--=========================================================
addParticle = function ( x, y, centerX, centerY, colorData ) 

	local particleFromElu = ParticleFromElu.new( world, centerX + x, centerY + y, colorData )

	particleFromElu.object.body:applyForce ( x*200, y*200 )
	
	particleList[#particleList + 1] = particleFromElu
	
	
end

--=========================================================
-- ずんだ餅を画面に出現させる
--=========================================================
addZunda = function () 
	
	local newZunda = Zunda.new ( world, math.random ( 64, map.w - 64 ), math.random ( 64, map.h - 64 ) )
	
	zundaList [ #zundaList + 1 ] = newZunda
	
end

--=========================================================
-- ずんだ餅を取る
--=========================================================
gotZunda = function ( zunda ) 

	--effectForScoreGettingNumber ( {value=1, x=light.object1.body:getX(), y=light.object1.body:getY() - 20} )

	local newZundaList = {}
	
	for i = 1, #zundaList, 1 do
		
		if ( zundaList[i] ~= zunda ) then
			newZundaList [ #newZundaList + 1 ] = zundaList[i]
		end
		
	end
	
	zundaList = newZundaList
	
	zunda:destruct()
	
	-- 体力満タンの場合は高得点
	if ( hp >= hpMax ) then
		
		local s = love.audio.newSource("power26.wav", "static")
		love.audio.play(s)
		
		score = score + 50
		effectForScoreGettingNumber ( {value=50, font = fontMiddle, x=objects.myChar.body:getX(), y=objects.myChar.body:getY() - 40} )
	
	-- 普通に回復
	else
		recoverToMyChar ( 20 )
	end
	
	
	
	--local s = love.audio.newSource("power18.wav", "static")
	--love.audio.play(s)
	
	--addLight(colorName)
end


--=========================================================
-- 光を画面に出現させる
--=========================================================
addLight = function ( color ) 
	
	--local lightColorList = {"red", "green", "blue"}
	--local lightColor = lightColorList[math.random(1,3)]
	
	local newLight = Light.new ( world, math.random ( 64, map.w - 64 ), math.random ( 64, map.h - 64 ), color) --lightColor )
	
	lightList [ #lightList + 1 ] = newLight
end

--=========================================================
-- 光を取る
--=========================================================
gotLight = function ( light ) 

	effectForScoreGettingNumber ( {value=1, x=light.object1.body:getX(), y=light.object1.body:getY() - 20} )

	
	local newLightList = {}
	
	for i = 1, #lightList, 1 do
		
		if ( lightList[i] ~= light ) then
			newLightList [ #newLightList + 1 ] = lightList[i]
		end
		
	end
	
	lightList = newLightList

	lightBox:putLight ( light.colorName )
	
	local colorName = light.colorName
	
	light:destruct()
	
	
	
	local s = love.audio.newSource("power18.wav", "static")
	love.audio.play(s)
	
	
	addLight(colorName)
end


--=========================================================
-- パーティクルを取る
--=========================================================
gotParticleHappened = false
gotParticle = function ( particleFromElu ) 
	
	gotParticleHappened = true
	
	score = score + 1
	effectForScoreGettingNumber ( {value=1, x=particleFromElu.object.body:getX(), y=particleFromElu.object.body:getY() - 20} )

	
	local newParticleList = {}
	
	for i = 1, #particleList, 1 do
		
		if ( particleList[i] ~= particleFromElu ) then
			newParticleList [ #newParticleList + 1 ] = particleList[i]
		end
		
	end
	
	particleList = newParticleList

	particleFromElu:destruct()
	
	
	
	local s = love.audio.newSource("coin03.wav", "static")
	love.audio.play(s)
	
end

--=========================================================
-- スコア獲得時の文字エフェクト
--=========================================================
effectForScoreGettingNumber = function (params) --( score, x, y, font ) 
	
	local score = params.value
	local x = params.x
	local y = params.y
	local font = params.font
	local color = params.color
	
	if ( color == nil ) then
		color = {r=1,g=1,b=0}
	end
	
	scoreGettingNumberEffectList [ #scoreGettingNumberEffectList + 1 ] = {
		x = x, y = y, score = score, frameCount = 0, font = font,color = color
	}
	
end
--=========================================================
-- ダメージ時の文字エフェクト
--=========================================================
effectForDamageGettingNumber = function ( params ) --damageValue, x, y, font ) 
	
	local damageValue = params.value
	local x = params.x
	local y = params.y
	local font = params.font
	local color = params.color
	
	if ( color == nil ) then
		color = {r=1,g=0,b=0}
	end
	
	damageGettingNumberEffectList [ #damageGettingNumberEffectList + 1 ] = {
		x = x, y = y, damageValue = damageValue, frameCount = 0, font = font, color = color
	}
	
end



--=========================================================
-- エルを消す
--=========================================================
deleteElu = function ( elu )
	
	local newEluList = {}
	
	for i = 1, #eluList, 1 do
		
		if ( eluList[i] ~= elu ) then
			newEluList [ #newEluList + 1 ] = eluList[i]
		end
		
	end
	
	eluList = newEluList
	
end


--=========================================================
-- エル落下
--=========================================================
eluDrops = function ( body ) 
	
	--local cx,cy,cz = love.audio.getPosition( )
	
	
	--love.audio.setPosition( -1,0,0 )
	
	local s = love.audio.newSource("hit45.wav", "static")
	
	local soundX = (body:getX() - objects.myChar.body:getX()) * 0.1
	--print(soundX)
	
	s:setPosition( soundX ,0,0 )
	
	love.audio.play(s)

	--love.audio.setPosition( cx, cy, cz )	
end



--=========================================================
-- 自キャラにダメージ
--=========================================================
damageToMyChar = function ( damageValue ) 

	hp = hp - damageValue
	if ( hp <= 0 ) then

		hp = 0
		
		resultWindow = ResultWindow.new()
	end
	
	damageCount = damageCount + damageValue
	
	effectForDamageGettingNumber( {value=damageValue, x=objects.myChar.body:getX() - 25, y=objects.myChar.body:getY() - 40} ) --, font )
	
	
	local s = love.audio.newSource("puu68.wav", "static")
	love.audio.play(s)
	
	
	releaseRope()

end

--=========================================================
-- 自キャラに回復
--=========================================================
recoverToMyChar = function ( recoverValue ) 

	hp = hp + recoverValue
	if ( hp > hpMax ) then
		hp = hpMax
	end
	
	effectForScoreGettingNumber( {
		value = recoverValue, x = objects.myChar.body:getX() - 25, y = objects.myChar.body:getY()-40, color={r=0,g=0,b=1} }
	) --, font )
	
	local s = love.audio.newSource("power06.wav", "static")
	love.audio.play(s)
	
end


--=========================================================
-- 自キャラとエルが接触
--=========================================================
collideWithElu = function ( elu ) 
	
	--local s = love.audio.newSource("hit45.wav", "static")
	--love.audio.play(s)
	
	
	
	-- ここはナシか。
	
	
end


--=========================================================
-- 衝突フィルター
--=========================================================
contactFilter = function ( fixtureA, fixtureB ) 
	
	local result = true
	

	local userDataA = fixtureA:getBody():getUserData()
	local userDataB = fixtureB:getBody():getUserData()	
	
	if ( userDataA == nil ) then userDataA = {obj=nil, name=""} end
	if ( userDataB == nil ) then userDataB = {obj=nil, name=""} end
	
	-- ワイヤーのアンカーは物理的な接触はしない
	if ( userDataA.name == "ropeAnchor" or userDataB.name == "ropeAnchor" ) then
		result = false
	end

	-- エルは天井には接触しない
	if ( userDataA.name == "elu1" and userDataB.name == "ceil" ) then result = false end
	if ( userDataA.name == "elu2" and userDataB.name == "ceil" ) then result = false end
	if ( userDataB.name == "elu1" and userDataA.name == "ceil" ) then result = false end
	if ( userDataB.name == "elu2" and userDataA.name == "ceil" ) then result = false end
	
	-- パーティクルも天井に接触しない
	if ( userDataA.name == "particleFromElu" and userDataB.name == "ceil" ) then result=false end
	if ( userDataB.name == "particleFromElu" and userDataA.name == "ceil" ) then result=false end


	-- エル落下
	if ( userDataA.name == "elu1" and userDataB.name == "ground" ) then eluDrops ( fixtureA:getBody() ) end
	if ( userDataA.name == "elu2" and userDataB.name == "ground" ) then eluDrops ( fixtureA:getBody() ) end
	if ( userDataB.name == "elu1" and userDataA.name == "ground" ) then eluDrops ( fixtureB:getBody() ) end
	if ( userDataB.name == "elu2" and userDataA.name == "ground" ) then eluDrops ( fixtureB:getBody() ) end
	
	-- 光には衝突判定はない。
	if ( userDataA.name == "light" or userDataB.name == "light" ) then
		result = false
	end
	-- パーティクルとずんだ餅には敢えて衝突判定は残しておく。ずんだの下でエルを防ぐ、という場面があるかもしれない。

	
	-- 以下、アイテム取得系
	
	if ( resultWindow == nil ) then
		
		-- 自キャラで光に接触
		if ( userDataA.name == "light" and userDataB.name == "myChar" ) then gotLight (userDataA.obj) end
		if ( userDataB.name == "light" and userDataA.name == "myChar" ) then gotLight (userDataB.obj) end

		-- 自キャラでパーティクルに接触
		if ( userDataA.name == "particleFromElu" and userDataB.name == "myChar" ) then gotParticle (userDataA.obj); result=false end
		if ( userDataB.name == "particleFromElu" and userDataA.name == "myChar" ) then gotParticle (userDataB.obj); result=false end

		-- 自キャラでずんだ餅に接触
		if ( userDataA.name == "zunda" and userDataB.name == "myChar" ) then gotZunda (userDataA.obj); result=false end
		if ( userDataB.name == "zunda" and userDataA.name == "myChar" ) then gotZunda (userDataB.obj); result=false end

		-- 自キャラでエルに接触
		if ( userDataA.name == "elu1" and userDataB.name == "myChar" ) then collideWithElu ( userDataA.obj ) end
		if ( userDataA.name == "elu2" and userDataB.name == "myChar" ) then collideWithElu ( userDataA.obj ) end
		if ( userDataB.name == "elu1" and userDataA.name == "myChar" ) then collideWithElu ( userDataA.obj ) end
		if ( userDataB.name == "elu2" and userDataA.name == "myChar" ) then collideWithElu ( userDataA.obj ) end

	end
	
	return result
end

--=========================================================
-- 衝突検知コールバック 
--=========================================================
function postSolve(fixture1, fixture2, contact, normal_impulse1, tangent_impulse1, normal_impulse2, tangent_impulse2)
	
	--print ( normal_impulse1, normal_impulse2 )
	
	local userData1 = fixture1:getBody():getUserData()
	local userData2 = fixture2:getBody():getUserData()	
	
	if ( userData1 == nil ) then userData1 = {obj=nil, name=""} end
	if ( userData2 == nil ) then userData2 = {obj=nil, name=""} end
	
	local isMyCharAndElu = false
	local normal_impulse_forMyChar = nil
	local tangent_impulse_forMyChar = nil
	if ( userData1.name == "myChar" and (userData2.name == "elu1" or userData2.name == "elu2") ) then
		isMyCharAndElu = true
		normal_impulse_forMyChar = normal_impulse2
		tangent_impulse_forMyChar = tangent_impulse2
	end
	if ( userData2.name == "myChar" and (userData1.name == "elu1" or userData1.name == "elu2") ) then
		isMyCharAndElu = true
		normal_impulse_forMyChar = normal_impulse1
		tangent_impulse_forMyChar = tangent_impulse1
	end
	
	-- 自キャラにエルが当たった場合
	if ( isMyCharAndElu ) then
	
		-- 前回のダメージのノックバック中は無敵
		if ( damageCount == 0 and resultWindow == nil) then 
		
			local nx, ny = contact:getNormal()
		
			--print ( "normal:", math.floor(nx*10), math.floor(ny*10) )
			--print ( normal_impulse_forMyChar, tangent_impulse_forMyChar )
		
			-- 下向きの圧力がかかっている（＝上から落ちてくるものに当たった、と想定）
			if ( ny > 0 ) then

				local impact = normal_impulse_forMyChar * ny
			
				local effectiveImpact = math.floor( impact*10)
			
				-- そこそこ大きい場合に限定
				if ( effectiveImpact > 1000 ) then
				
					local damage = math.floor ( effectiveImpact/200 )
					damageToMyChar ( damage )
				
					--addSystemMessage ("落ちてくるエルに当たった：" .. effectiveImpact )
				end
			end
			
		end
		
	end
end













