local utils = require "utils"
require("button")
require("featWindow")
require("instructionWindow")

TitleScene = {}
TitleScene.new = function ()
	
	local this = {}
	
	this.id = "titleScene"
	this.canvas = love.graphics.newCanvas()
	
	this.buttonStart = Button.new( fonts.system, "スタート")
	this.buttonStart.width  = 160
	this.buttonStart.height = 100
	this.buttonStart.x = screenWidth/2 - 180
	this.buttonStart.y = screenHeight - 100

	this.buttonFeat = Button.new( fonts.system, "称号")
	this.buttonFeat.width  = 160
	this.buttonFeat.height = 100
	this.buttonFeat.x = this.buttonStart.x + 180
	this.buttonFeat.y = screenHeight - 100
	
	this.buttonHelp = Button.new( fonts.system, "説明")
	this.buttonHelp.width  = 160
	this.buttonHelp.height = 100
	this.buttonHelp.x = this.buttonFeat.x + 180
	this.buttonHelp.y = screenHeight - 100

	
	this.logoImage = love.graphics.newImage ( "titleLogo.png" )
	this.charImage = love.graphics.newImage ( "seiwink_b_600_600.png" )
	this.backgroundImage = love.graphics.newImage ( "titleBackground.png" )
	
	this.charImagePos = {}--{x = screenWidth, y = 0}
	this.logoImagePos = {}--{x = -screenWidth, y = 0}
    
	--love.filesystem.setIdentity("screenshot_example")
	
	
	featWindow = nil
	instructionWindow = nil
	
	--=========================================================
	-- 更新
	--=========================================================
	this.update = function(self, dt)
		
		if ( featWindow ~= nil ) then
			local result = featWindow:update()
			if ( result == "done" ) then
				featWindow = nil
			end
			return
		end
		
		if ( instructionWindow ~= nil ) then
		
			local result = instructionWindow:update()
			if ( result == "done" ) then
				instructionWindow = nil
			end
			return
		end
		
		
		this.buttonStart:update()
		this.buttonFeat:update()
		this.buttonHelp:update()
		
		if (self.buttonStart.isClicked) then
			sceneManager.nextScene = "play"
		end
		if (self.buttonFeat.isClicked) then
			featWindow = FeatWindow.new()
		end
		if (self.buttonHelp.isClicked) then
			instructionWindow = InstructionWindow.new()
		end
		
		this.charImagePos.x = utils.Approach ( this.charImagePos.x, screenWidth - 500, 30 )
		this.logoImagePos.x = utils.Approach ( this.logoImagePos.x, 0, 30 )
		
		
		if love.keyboard.isDown ("return" ) then
			
			-- テスト用
			
			--love.graphics.captureScreenshot(os.time() .. ".png")
			
			--utils.writeFile ( os.time() .. ".txt", "書き込みテスト" ) 
		
		end
		
	end
	
	--=========================================================
	-- 描画
	--=========================================================
	this.draw = function(self)
		
		local r,g,b,a = love.graphics.getColor()
		--local currentCanvas = love.graphics.getCanvas()
		
		--love.graphics.setCanvas ( self.canvas )

		love.graphics.clear(1,1,1,1)
		
		--love.graphics.draw ( titleBack, 0,0 )
		
		
		
		love.graphics.draw ( this.backgroundImage, 0, 0  )

		love.graphics.draw ( this.charImage, this.charImagePos.x, this.charImagePos.y  )

		love.graphics.draw ( this.logoImage, this.logoImagePos.x, this.logoImagePos.y  )
		
		
		-- なんか、シェーダーがあるとボタンが真っ白になるので。
		if ( sceneManager.shader == nil ) then
			self.buttonStart:draw()
			self.buttonFeat:draw()
			self.buttonHelp:draw()
		end
		
		
		--love.graphics.setCanvas ( currentCanvas )

		--love.graphics.draw ( self.canvas, 0, 0 )

		
		-- 功績ウィンドウ
		if ( featWindow ~= nil ) then
			featWindow:draw()
		end
		
		-- 説明ウィンドウ
		if ( instructionWindow ~= nil ) then
			instructionWindow:draw()
		end

		love.graphics.setColor(r,g,b,a)
	end
	
	
	
	--=========================================================
	-- シーン開始
	--=========================================================
	this.enter = function(self)
		print("開始："..self.id)
		
		this.charImagePos = {x = screenWidth, y = 0}
		this.logoImagePos = {x = -screenWidth + 140, y = 0}
		
		love.audio.play ( bgm.title )
		
	end
	--=========================================================
	-- シーン終了
	--=========================================================
	this.leave = function(self)
		print("終了："..self.id)

		love.audio.stop ( bgm.title )
	end
	
	
	
	return this
end
